/*
    cc1100.c -
    Copyright (C) 2007

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */

#include "lpc2220.h"
#include "cc1100.h"
#include "irq.h"


/* ================== Datentypen und Variablendefinitionen ================= */

//setting 6_WOR
const unsigned char conf[0x2F] = {
    0x29 ,     //IOCFG2
    0x2E ,     //IOCFG1
    0x06 ,     //IOCFG0
    0x47 ,     //FIFOTHR
    0xD3 ,     //SYNC1
    0x91 ,     //SYNC0
    0x3E ,     //PKTLEN
    0x1A ,     //PKTCTRL1
    0x45 ,     //PKTCTRL0
    0x01 ,     //ADDR
    0x01 ,     //CHANNR
    0x06 ,     //FSCTRL1
    0x00 ,     //FSCTRL0
    0x10 ,     //FREQ2
    0x0B ,     //FREQ1
    0xDA ,     //FREQ0
    0x8A ,     //MDMCFG4
    0x75 ,     //MDMCFG3
    0x13 ,     //MDMCFG2
    0x22 ,     //MDMCFG1
    0xC1 ,     //MDMCFG0
    0x35 ,     //DEVIATN
    0x04 ,     //MCSM2
    0x0C ,     //MCSM1 0c
    0x38 ,     //MCSM0
    0x16 ,     //FOCCFG
    0x6C ,     //BSCFG
    0x43 ,     //AGCCTRL2
    0x40 ,     //AGCCTRL1
    0x91 ,     //AGCCTRL0
    0x46 ,     //WOREVT1
    0x50 ,     //WOREVT0
    0x78 ,     //WORCTRL
    0x56 ,     //FREND1
    0x10 ,     //FREND0
    0xA9 ,     //FSCAL3
    0x0A ,     //FSCAL2
    0x00 ,     //FSCAL1
    0x11 ,     //FSCAL0
    0x41 ,     //RCCTRL1
    0x00 ,     //RCCTRL0
    0x57 ,     //FSTEST
    0x7F ,     //PTEST
    0x3F ,     //AGCTEST
    0x98 ,     //TEST2
    0x31 ,     //TEST1
    0x0B     //TEST0
};

const unsigned char confasync[0x2F] = {
    0x0D ,     //IOCFG2
    0x0D ,     //IOCFG1
    0x2E ,     //IOCFG0
    0x47 ,     //FIFOTHR
    0xD3 ,     //SYNC1
    0x91 ,     //SYNC0
    0x3E ,     //PKTLEN
    0x1A ,     //PKTCTRL1
    0x32 ,     //PKTCTRL0
    0x00 ,     //ADDR
    0x00 ,     //CHANNR
    0x06 ,     //FSCTRL1
    0x00 ,     //FSCTRL0
    0x10 ,     //FREQ2    433,919861MHz
    0x12 ,     //FREQ1
    0x34 ,     //FREQ0
    0x4A ,     //MDMCFG4
    0x84 ,     //MDMCFG3
    0x30 ,     //MDMCFG2
    0x20 ,     //MDMCFG1
    0xe5 ,     //MDMCFG0    Chann 49,95kHz
    0x37 ,     //DEVIATN
    0x07 ,     //MCSM2
    0x30 ,     //MCSM1 0c
    0x18 ,     //MCSM0
    0x14 ,     //FOCCFG
    0x6C ,     //BSCFG
    0x07 ,     //AGCCTRL2
    0x00 ,     //AGCCTRL1
    0x90 ,     //AGCCTRL0
    0x46 ,     //WOREVT1
    0x50 ,     //WOREVT0
    0x78 ,     //WORCTRL
    0x56 ,     //FREND1
    0x11 ,     //FREND0
    0xE9 ,     //FSCAL3
    0x2A ,     //FSCAL2
    0x00 ,     //FSCAL1
    0x1F ,     //FSCAL0
    0x41 ,     //RCCTRL1
    0x00 ,     //RCCTRL0
    0x59 ,     //FSTEST
    0x7F ,     //PTEST
    0x3F ,     //AGCTEST
    0x81 ,     //TEST2
    0x35 ,     //TEST1
    0x09     //TEST0
};


/* ======================= Funktionen und Programmcode ===================== */

/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_init
   ----------------------------------------------------------------------------
*/
void cc1100_init (void)
{

   /* ----- lokale Variablen ----- */
    unsigned long xx = 0x200;


   /* ----- Programmcode ----- */
    PINSEL1 &= 0xffffff00;                    //GDO0 SCK1 MISO1 MOSI1 as GPIO
    FIODIR0 |= (CS1 | MOSI1 | SCK1);         //output

    PCONP &= 0xfffffbff;
    PCONP |= (1<<21);
    FIOSET0 = SCK1;
    FIOCLR0 = MOSI1;
    FIOCLR0 = CS1;

    while (xx)
    {
        xx--;
    }

    FIOSET0 = CS1;
    xx=0x200;

    while (xx)
    {
        xx--;
    }

    FIOCLR0 = CS1;
    FIOCLR0 = SCK1;
    while (FIOPIN0 & MISO1);

    PINSEL1 |= 0x000002A8;        //SCK1 MISO1 MOSI1 as SPI1
    SSPCR0 = 0x0007;
    SSPCPSR = 0x02;
    SSPCR1 = 0x02;

    SSPDR = SRES;
    while (FIOPIN0 & MISO1);
    while (SSPSR & (1<<4));
    xx = SSPDR;

    cc1100_write ((0x00 | BURST ), (unsigned char*)conf,0x2f); // Parameter: unsigned char addr, unsigned char* data, unsigned char length
    cc1100_write1 (PATABLE,0xC0);                              // Parameter: unsigned char addr,unsigned char data
    cc1100_strobe (SIDLE);
    cc1100_strobe (SPWD);
}
/* ----- Ende cc1100_init -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_write
   ----------------------------------------------------------------------------
*/
unsigned char cc1100_write (unsigned char addr,unsigned char* data, unsigned char length)
{

   /* ----- lokale Variablen ----- */
    unsigned short i;
    unsigned char status;
    unsigned char x;


   /* ----- Programmcode ----- */
    FIOCLR0 = CS1;
    while (FIOPIN0 & MISO1);
    SSPDR = (addr | WRITE);
    while ((SSPSR & (1<<4)));
    status = SSPDR;
    for (i=0; i < length; i++)
    {
        SSPDR = data[i];
        while ((SSPSR & (1<<4)));
        x=SSPDR;
    }
    FIOSET0 = CS1;

    return (status);
}
/* ----- Ende cc1100_write ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_write1
   ----------------------------------------------------------------------------
*/
unsigned char cc1100_write1 (unsigned char addr,unsigned char data)
{

   /* ----- lokale Variablen ----- */
    unsigned char status;
    unsigned char x;


   /* ----- Programmcode ----- */
    FIOCLR0 = CS1;
    while (FIOPIN0 & MISO1);
    SSPDR = (addr | WRITE);
    while ((SSPSR & (1<<4)));
    status = SSPDR;

    SSPDR = data;
    while ((SSPSR & (1<<4)));
    x=SSPDR;

    FIOSET0 = CS1;

    return (status);
}
/* ----- Ende cc1100_write1 ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_read
   ----------------------------------------------------------------------------
*/
unsigned char cc1100_read (unsigned char addr, unsigned char* data, unsigned char length)
{

   /* ----- lokale Variablen ----- */
    unsigned short i;
    unsigned char status;


   /* ----- Programmcode ----- */
    FIOCLR0 = CS1;

    while (FIOPIN0 & MISO1);
    SSPDR = (addr | READ);
    while ((SSPSR & (1<<4)));
    status = SSPDR;
    for (i=0; i < length; i++) {
        SSPDR = 0x00;
        while ((SSPSR & (1<<4)));
        data[i]=SSPDR;
    }
    FIOSET0 = CS1;

    return (status);
}
/* ----- Ende cc1100_read -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_read1
   ----------------------------------------------------------------------------
*/
unsigned char cc1100_read1 (unsigned char addr)
{

   /* ----- lokale Variablen ----- */
    unsigned char r;


   /* ----- Programmcode ----- */
    FIOCLR0 = CS1;
    while (FIOPIN0 & MISO1);
    SSPDR = (addr | READ);
    while ((SSPSR & (1<<4)));
    r = SSPDR;
    SSPDR = 0x00;
    while ((SSPSR & (1<<4)));
    r=SSPDR;
    FIOSET0 = CS1;

    return (r);
}
/* ----- Ende cc1100_read1 ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cc1100_strobe
   ----------------------------------------------------------------------------
*/
unsigned char cc1100_strobe (unsigned char cmd)
{

   /* ----- lokale Variablen ----- */
    unsigned short status;


   /* ----- Programmcode ----- */
    FIOCLR0 = CS1;
    while (FIOPIN0 & MISO1);
    SSPDR = cmd;
    while ((SSPSR & (1<<4)));
    status = SSPDR;
    FIOSET0 = CS1;

    return (status);
}
/* ----- Ende cc1100_strobe ------------------------------------------------ */


