 /*
    rf.c -
    Copyright (C) 2008

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */

#include <string.h>

#include "lpc2220.h"
#include "rf.h"
#include "cc1100.h"
#include "timerfuncs.h"
#include "irq.h"
#include "cc1100.h"
#include "lcd.h"
#include "fonty.h"
#include "rtc.h"
#include "global.h"
#include "buffer.h"
#include "bfs.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#define RFSTARLIGHT_DEBUG  0

#if RFSTARLIGHT_DEBUG
   #include "serial.h"
   char trace_string[255];
#endif

unsigned char RFbuf [64];
struct RFendpoint_ rfendpoint [MAX_ENDPOINTS];
struct RF_ RF;

const struct RFsettings_ default_RFsettings =
{
   0x01,   //address
   0x01    //channel
};



/* ======================= Funktionen und Programmcode ===================== */

//*********************
//* Private functions *
//*********************

/*
   ----------------------------------------------------------------------------
   Funktion: switch_to_idle
   ----------------------------------------------------------------------------
*/
void switch_to_idle ()
{
   /* ----- Programmcode ----- */
   cc1100_strobe (SIDLE);
   while (cc1100_read1 (MARCSTATE) != 01);
}
/* ----- Ende switch_to_idle ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: WORsend_timeout
   ----------------------------------------------------------------------------
*/
void WORsend_timeout (void)
{
   /* ----- Programmcode ----- */
   RF.state = RFsendworend;
}
/* ----- Ende WORsend_timeout ---------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: sendendpoint
   ----------------------------------------------------------------------------
*/
void sendendpoint (struct RFendpoint_ *currentEP)
{
   /* ----- lokale Variablen ----- */
   unsigned char* epdata = 0;
   unsigned short eplen;
   struct cc1100frame_ *TXframe;


   /* ----- Programmcode ----- */
   TXframe = (struct cc1100frame_ *)RFbuf;   // RFbuf is one global buffer

   //write header
   eplen = currentEP->bufferlen;
   switch (currentEP->flags & EPtypemask)
   {
      case EPtypedata:
         if (eplen > 6)
            eplen = 6;
         epdata = currentEP->data;

         currentEP->flags &= ~EPnewdata;
      break;

      case EPtypebuffer:
         if (eplen > 59)
            eplen = 59;
         epdata = currentEP->buffer;

         // Rest of the buffer will be discarded, use EPtypecbuffer to avoid this problem
         currentEP->flags &= ~EPnewdata;
      break;

      case EPtypecbuffer:
         // If cBuffer_t is used, the len is already defined there, so we ignore currentEP->bufferlen.
         eplen = currentEP->cbuffer->len;
         if (eplen > 59)
            eplen = 59;

         if (eplen == currentEP->cbuffer->len)
            currentEP->flags &= ~EPnewdata;
      break;
   }   // switch (currentEP->flags & EPtypemask)

   TXframe->len = eplen + 3;
   TXframe->destAddr = currentEP->dest;
   TXframe->srcAddr = RF.addr;
   TXframe->packetType = currentEP->type;
   cc1100_write (TX_fifo | BURST, (unsigned char*)TXframe,4);

   //write data
   switch (currentEP->flags & EPtypemask)
   {
      case EPtypedata:
      case EPtypebuffer:
         cc1100_write (TX_fifo | BURST, epdata, eplen);
      break;
      case EPtypecbuffer:
         if (currentEP->cbuffer->index + eplen <= currentEP->cbuffer->size)
         {
            cc1100_write (TX_fifo | BURST,
                          &(currentEP->cbuffer->data [currentEP->cbuffer->index]),
                          eplen);
         }
         else
         {
            cc1100_write (TX_fifo | BURST,
                          &(currentEP->cbuffer->data [currentEP->cbuffer->index]),
                          currentEP->cbuffer->size - currentEP->cbuffer->index);
            cc1100_write (TX_fifo | BURST,
                          currentEP->cbuffer->data,
                          eplen - (currentEP->cbuffer->size - currentEP->cbuffer->index));
         }
         currentEP->cbuffer->index = (currentEP->cbuffer->index + eplen) % currentEP->cbuffer->size;
         currentEP->cbuffer->len -= eplen;
      break;
   }

   cc1100_strobe (STX);

   if (currentEP->flags & EPonce)
      closeEP (currentEP);
}   // sendendpoint
/* ----- Ende sendendpoint ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: receiveendpoint
   ----------------------------------------------------------------------------
*/
/// This function is called by the ISR
void receiveendpoint (struct RFendpoint_ *currentEP)
{
   /* ----- lokale Variablen ----- */
   unsigned short eplen;
   struct cc1100frame_ *RXframe;
   #if RFSTARLIGHT_DEBUG
      char trace_string[255];
      unsigned char index;
   #endif


   /* ----- Programmcode ----- */
   RXframe = (struct cc1100frame_ *)RFbuf;   // RFbuf is one global buffer

   #if RFSTARLIGHT_DEBUG
      sprintf ( trace_string , "Adresse von currentEP : %x\n", (void *)currentEP ); serial_puts( trace_string );
      sprintf ( trace_string , "RX len        : %x\n", RXframe->len );              serial_puts( trace_string );
      sprintf ( trace_string , "RX destAddr   : %x\n", RXframe->destAddr );         serial_puts( trace_string );
      sprintf ( trace_string , "RX srcAddr    : %x\n", RXframe->srcAddr );          serial_puts( trace_string );
      sprintf ( trace_string , "RX packetType : %x\n", RXframe->packetType );       serial_puts( trace_string );
   #endif

   currentEP->dest = RXframe->srcAddr;
   eplen = ( RXframe->len )-3;

   #if RFSTARLIGHT_DEBUG
      sprintf ( trace_string , "eplen : %x\n", eplen );
      serial_puts( trace_string );
   #endif

   switch (currentEP->flags & EPtypemask)
   {
      case EPtypedata:
         if (eplen > 6)
         {
            eplen = 6;
            currentEP->flags |= EPoverflow;
         }
         memcpy (& (currentEP->data[0]), RXframe->data, eplen);
      break;

      case EPtypebuffer:

         #if RFSTARLIGHT_DEBUG
            serial_puts( "Korrekter Buffertype erkannt" );
         #endif

         if (eplen > currentEP->buffersize)
         {
            eplen = currentEP->buffersize;
            currentEP->flags |= EPoverflow;
         }
         memcpy (currentEP->buffer, RXframe->data, eplen);

         #if RFSTARLIGHT_DEBUG
            serial_puts( "\nreceiveendpoint: Ausgabe empfangener Bytes:\n" );
            for ( index = 0 ; index < eplen ; index++ )
            {
               sprintf ( trace_string , "%x ", currentEP->buffer[ index ] ); serial_puts( trace_string );
            }
            serial_puts( "\n" );
         #endif

      break;

      case EPtypecbuffer:
         if (eplen > currentEP->cbuffer->size - currentEP->cbuffer->len)
         {
            eplen = currentEP->cbuffer->size - currentEP->cbuffer->len;
            currentEP->flags |= EPoverflow;
         }
         cBufferAddBuffer (currentEP->cbuffer, RXframe->data, eplen);
      break;
   }

   // Nun noch korrekte Lnge in Datenstruktur eintragen
   currentEP->bufferlen = eplen;

   if (currentEP->flags & EPnewdata)
   {
      currentEP->flags |= EPoverwritten;
   }
   else
   {
      currentEP->flags |= EPnewdata;
   }

   // Achtung: Wurde flag ONCE gesetzt wird anschlieen Datenstruktur "abgeschossen"
   if (currentEP->flags & EPonce)
      closeEP (currentEP);

}   // receiveendpoint
/* ----- Ende receiveendpoint ---------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion:
   ----------------------------------------------------------------------------
*/
void cc1100IRQ (void)
{
   /* ----- lokale Variablen ----- */
   unsigned char x;


   /* ----- Programmcode ----- */
   switch_to_idle ();

   if (cc1100_read1 (RXBYTES) > 0)
   {
      struct cc1100frame_ *RXframe;

      cc1100_read (RX_fifo, RFbuf, 1);
      cc1100_read (RX_fifo, &RFbuf[1], RFbuf[0]);

      RXframe = (struct cc1100frame_ *)RFbuf;

      if ((RXframe->len == 1) &&
          (RF.flags & WORenabled) &&
          ((RXframe->destAddr == RF.addr) ||
          (RXframe->destAddr == 0x00)))
      {
         RF_changestate (RFrx);
         // serial_puts( "und jetzt??" );
         //if (RF.timeout_cb != 0xff || RF.state != RFrx)
            // RF_timed_rx (WORrx_time);
      }

      if (RXframe->len > 3)
      {

         #if RFSTARLIGHT_DEBUG
            sprintf ( trace_string , "RXframe->len        : %x\n", RXframe->len );        serial_puts( trace_string );
            sprintf ( trace_string , "RXframe->destAddr   : %x\n", RXframe->destAddr );   serial_puts( trace_string );
            sprintf ( trace_string , "RXframe->srcAddr    : %x\n", RXframe->srcAddr );    serial_puts( trace_string );
            sprintf ( trace_string , "RXframe->packetType : %x\n", RXframe->packetType ); serial_puts( trace_string );
         #endif

         if (RF.timeout_cb != 0xff)
            setCBIntervall (RF.timeout_cb, WORrx_time);

         switch (RXframe->packetType)
         {
            /* ===== RF_PACKET_TEST ===== */
            case RF_PACKET_TEST:
               draw_block  (0, 50, 128, 10, LCD_COLOR_B, DRAW_ERASE);
               set_font (SMALLFONT);
               draw_string (0, 51, (char*)RXframe->data, LCD_COLOR_B, DRAW_PUT);
            break;

            /* ===== RF_PACKET_PING ===== */
            case RF_PACKET_PING:
               if (RXframe->data[0] == 0x01)
               {
                  struct RFendpoint_* cur_ep;
                  cur_ep = openEP (0, 0, RF_PACKET_PING);
                  if (cur_ep)
                  {
                     cur_ep->dest = RXframe->srcAddr;
                     cur_ep->data[0] = 2;
                     cur_ep->bufferlen = 1;
                     cur_ep->flags |= EPenabled | EPoutput | EPnewdata | EPonce;
                     RF_changestate (RFtx);
                  }
               }
               else if (RXframe->data[0] == 0x02)
               {
                  RF.flags |= PingACK;
               }
            break;

            /* ===== RF_PACKET_TIME ===== */
            case RF_PACKET_TIME:
               if ((RXframe->data[0] == 0xFF) && (timeInfo & time_accurate))
               {
                  // Received a get time request
                  send_time (RXframe->srcAddr,0);
               }
               else if (RXframe->len == 10)
               {
                  // Received a new time info
                  memcpy ((unsigned char *)&rtc, &RXframe->data[0], 7);
                  timeInfo |= setnewtime;
                  VICSoftInt = INT_EINT2;
               }
            break;

            /* ===== default ===== */
            default:
               for (x = 0; x < MAX_ENDPOINTS; x++)
               {
                  if ((rfendpoint [x].type == RXframe->packetType) &&
                      // ... and it is enabled but NOT a output endpoint
                      ((rfendpoint [x].flags & (EPoutput | EPenabled)) == EPenabled))
                  {
                     #if RFSTARLIGHT_DEBUG
                        serial_puts( "++ Datenpaket empfangen\n" );
                     #endif

                     receiveendpoint (&rfendpoint [x]);
                     break;
                  }
               }
            break;
         }   // switch (RXframe->packetType)
      }   // if (RXframe->len > 3)
   }   // if (cc1100_read1 (RXBYTES) > 0)

   if (RF.state == RFtx)
   {
      for (x = 0; x < MAX_ENDPOINTS; x++)
      {
         if ((rfendpoint[x].type) &&
             ((rfendpoint[x].flags & (EPoutput | EPnewdata | EPenabled)) == (EPoutput | EPnewdata | EPenabled)))
         {
            if (rfendpoint[x].flags & EPsendwor)
            {
               rfendpoint[x].flags &= ~EPsendwor;
               cc1100_write1 (MCSM0, conf[MCSM0] & 0xCF);
               cc1100_strobe (SCAL);
               while (cc1100_read1 (MARCSTATE) != 01);
               RF.wortimeout_cb = addTimerCB (WORsend_timeout, 101);
               startCB (RF.wortimeout_cb);
               RF.state = RFsendwor;
               RF.tag = x;   // save the EPsendwor-endpoint in RF.tag
            }
            else
            {
               sendendpoint (&rfendpoint[x]);
            }

            break;
         }
      }
      if (x == MAX_ENDPOINTS)
      {
         RF.state = RF.nextstate;
      }
   }   // if (RF.state == RFtx)

   if (RF.state == RFsendwor)
   {
      unsigned char b[2];
      b[0] = 0x01;
      b[1] = rfendpoint [RF.tag].dest;   // EPsendwor-endpoint is stored in RF.tag
      cc1100_write (TX_fifo | BURST,b,2);
      cc1100_strobe (STX);
   }   // if (RF.state == RFsendwor)

   if (RF.state == RFsendworend)
   {
      stopCB (RF.wortimeout_cb);
      removeTimerCB (RF.wortimeout_cb);
      RF.wortimeout_cb = 0xff;

      cc1100_write1 (MCSM0,conf[MCSM0]);
      RF.state = RFtx;
      sendendpoint (&rfendpoint[RF.tag]);
   }   // if (RF.state == RFsendworend)

   switch (RF.state)
   {
      case RFpwrdown:
         cc1100_strobe (SPWD);
      break;

      case RFwor:
         if (! (RF.flags & RXueop))
         {
            cc1100_write1 (0x16,conf[0x16]);
            RF.flags |= RXueop;
         }
         cc1100_strobe (SWOR);
      break;

      case RFidle:
      break;

      case RFrx:
         if (RF.flags & RXueop)
         {
            cc1100_write1 (0x16,0x07);
            RF.flags &= ~RXueop;
         }
         cc1100_strobe (SFRX);
         cc1100_strobe (SRX);
      break;

      case RFtx:
         break;

      case RFsendwor:
         break;
   }

   if (RF.state <= RFidle)
      symbols &= ~symbolRF;
   else
      symbols |= symbolRF;

   VICSoftIntClr = INT_EINT0;
   EXTINT = 0x01;
}   // cc1100IRQ
/* ----- Ende cc1100IRQ ---------------------------------------------------- */


//********************
//* Public functions *
//********************

/*
   ----------------------------------------------------------------------------
   Funktion: RF_init
   ----------------------------------------------------------------------------
*/

void RF_init (void)
{
   /* ----- Programmcode ----- */
   cc1100_init ();
   RF.flags = 0;
   RF.addr = conf[0x09];
   RF.state = RFpwrdown;
   RF.nextstate = RFpwrdown;
   RF.timeout_cb = 0xff;
}
/* ----- Ende RF_init ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: RF_changestate
   ----------------------------------------------------------------------------
*/
void RF_changestate (unsigned char state)
{
   /* ----- lokale Variablen ----- */
   unsigned char oldstate;


   /* ----- Programmcode ----- */
   VICIntEnClr = INT_EINT0;   // stop RF IRQ

   if (RF.state != state)
   {
      oldstate = RF.state;

      if ((RF.state >= RFtx) && (state < RFtx))
      {
         // RX or Idle is requested while TX is running => Queue the request
         RF.nextstate = state;
      }
      else
      {
         switch (state)
         {
            case RFpwrdown:
               RF.state = RFpwrdown;
               break;

            case RFwor:
               RF.state = RFwor;
               RF.flags |= WORenabled;
               break;

            case RFidle:
               RF.state = RFidle;
               break;

            case RFrx:
               RF.state = RFrx;
               break;

            case RFtx:

               // If currently not transmitting
               if (RF.state < RFtx)
               {
                  RF.state = RFtx;
               }

               // If it was receiving before, keep receiving after TX
               if (oldstate == RFrx)
               {
                  RF.nextstate = RFrx;
               }
               else if (RF.flags & WORenabled)
               {
                  RF.nextstate = RFwor;
               }
               else
               {
                  RF.nextstate = RFidle;
               }
               break;

            case RFsendwor:
               break;
         }
      }
      if (oldstate != RF.state)
      {
         VICSoftInt = INT_EINT0;
      }
   }
   VICIntEnable = INT_EINT0;   // enable RF IRQ
}   // RF_changestate
/* ----- Ende RF_changestate ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: startRFIRQ
   ----------------------------------------------------------------------------
*/
void startRFIRQ (void)
{
   /* ----- Programmcode ----- */
   PINSEL1  |= 1;        //GDO0 as EINT0
   EXTMODE  |= 1;        //edge sensitive
   EXTPOLAR |= 0;        //falling edge
   EXTWAKE  |= 1;
   EXTINT   |= 1;

   VICVectAddr2 = (unsigned long)& (cc1100IRQ);
   VICVectCntl2 = VIC_SLOT_EN | INT_SRC_EINT0;
   VICIntEnable = INT_EINT0;
}   // startRFIRQ
/* ----- Ende startRFIRQ --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: openEP
   ----------------------------------------------------------------------------
*/
struct RFendpoint_* openEP (void* buffer, unsigned short buffersize, unsigned char type)
{

   /* ----- lokale Variablen ----- */
   int i;
   struct RFendpoint_* cur_ep;


   /* ----- Programmcode ----- */
   if (!type)
      return 0;

   for (i=0; i<MAX_ENDPOINTS;i++)
   {
      cur_ep = &rfendpoint[i];
      if (cur_ep->type == RF_PACKET_NONE)
      {
         cur_ep->buffer = buffer;
         cur_ep->buffersize = buffersize;
         cur_ep->type = type;
         cur_ep->bufferlen = 0;
         cur_ep->dest = 0;
         cur_ep->flags = 0;
         if (! (buffer))
            cur_ep->flags |= EPtypedata;
         return cur_ep;
      }
   }
   return 0;
}   // openEP
/* ----- Ende openEP ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: closeEP
   ----------------------------------------------------------------------------
*/
void closeEP (struct RFendpoint_ *cur_ep)
{
   /* ----- Programmcode ----- */
   if (cur_ep)
   {
      cur_ep->flags  = 0;
      cur_ep->buffer = 0;
      cur_ep->type   = RF_PACKET_NONE;
   }
}
/* ----- Ende closeEP ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: RF_setRegister
   ----------------------------------------------------------------------------
*/
void RF_setRegister (unsigned char reg, unsigned char data)
{
   /* ----- lokale Variablen ----- */
   unsigned long VICInt;


   /* ----- Programmcode ----- */
   VICInt = VICIntEnable & INT_EINT0;
   VICIntEnClr = INT_EINT0;

   cc1100_write1 (reg,data);

   VICIntEnable |= VICInt;
}
/* ----- Ende RF_setRegister ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: load_RF_setting
   ----------------------------------------------------------------------------
*/
void load_RF_setting (void)
{
   /* ----- lokale Variablen ----- */
   struct RFsettings_ RFsettings;


   /* ----- Programmcode ----- */
   if ( BFS_LoadFile (BFS_ID_RFsettings, sizeof (struct RFsettings_), (unsigned char*) &RFsettings) != sizeof (struct RFsettings_) )
   {
      RFsettings = default_RFsettings;
      BFS_SaveFile (BFS_ID_RFsettings, sizeof (struct RFsettings_), (unsigned char*) &RFsettings);
      draw_string (0, 50, "default RF load", LCD_COLOR_B, DRAW_PUT);
   }

   RF_setRegister (0x09, RFsettings.address);
   RF_setRegister (0x0a, RFsettings.channel);
   RF.addr = RFsettings.address;

}
/* ----- Ende load_RF_setting ---------------------------------------------- */


