/*
    rf.h -
    Copyright (C) 2008

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


/* ================== Datentypen und Variablendefinitionen ================= */

#ifndef RF_H
#define RF_H

#define _BV(bit)   (1 << (bit))

// #define WORrx_time            200
#define WORrx_time            400

#define RF_PACKET_NONE        0
#define RF_PACKET_TEST        1
#define RF_PACKET_PING        2
#define RF_PACKET_TIME        3
#define RF_PACKET_RF_ENC      4
#define RF_PACKET_REDIR_DAT   5
#define RF_PACKET_REDIR_CMD   6
#define RF_PACKET_XML         7
#define RF_PACKET_BINARY      8


#define MAX_ENDPOINTS         0x0F

struct cc1100frame_
{
   unsigned char len;
   unsigned char destAddr;
   unsigned char srcAddr;
   unsigned char packetType;
   unsigned char data[];
};

typedef struct RFendpoint_
{
   /// three possible buffer configurations
   union
   {
      unsigned char*    buffer;
      struct cBuffer_*  cbuffer;
      unsigned char     data[4];
   };

   unsigned short             buffersize; // maximal amount of bytes in buffer above
   unsigned short             bufferlen;  // count of receved bytes or bytes to sent in the buffer
   volatile unsigned short    flags;      // Enabled, In/Out, buffer type, new data, overflow
   unsigned char              dest;
   unsigned char              type;       // RF_PACKET_xxx

} RFendpoint_t, *RFendpoint_p;


///RFendpoints_.flags
#define EPenabled       _BV(0)
#define EPoutput        _BV(1)
#define EPinput         0
#define EPtypedata      _BV(2)
#define EPtypebuffer    0
#define EPtypecbuffer   _BV(3)
#define EPtypemask     (_BV(2) | _BV(3))
#define EPnewdata       _BV(4)
#define EPoverwritten   _BV(5)
#define EPoverflow      _BV(6)
#define EPonce          _BV(7)
#define EPsendwor       _BV(8)  // Wake On Radio


struct RFsettings_
{
   unsigned char address;
   unsigned char channel;
};

// const struct RFsettings_ default_RFsettings;

struct RF_
{
   unsigned char addr;
   volatile unsigned char flags;
   volatile unsigned char state;
   volatile unsigned char timeout_cb;
   volatile unsigned char wortimeout_cb;
   volatile unsigned char nextstate;
   volatile unsigned char tag;
};
struct RF_ RF;

/// RF_.state
#define RFpwrdown      0
#define RFwor          1   // Wake On Radio
#define RFidle         2
#define RFrx           3
/// tx states need to be after RFtx
#define RFtx           4
#define RFsendwor      5   // Wake On Radio
#define RFsendworend   6

/// RF.flags
#define PingACK        _BV(0)
#define WORenabled     _BV(1)   // Wake On Radio
#define RXueop         _BV(2)   // MCSM2 RX_TIME: Until end of packet


/* ======================== Prototyping ==================================== */

void           startRFIRQ (void);
void           RF_init (void);
void           RF_changestate (unsigned char state);
unsigned char  RF_sendping (unsigned char dest);
struct         RFendpoint_* openEP (void* buffer, unsigned short buffersize, unsigned char type);
void           closeEP (struct RFendpoint_ *cur_ep);
void           RF_setRegister (unsigned char reg, unsigned char data);
void           load_RF_setting (void);
// void RF_timed_rx (unsigned short timeout);
// void RFasyncmode (unsigned char on);

#endif

/* =================================== Ende ================================ */


