/*
    bfs.c - Betty File System
    Copyright (C) 2007  Colibri <colibri_dvb@lycos.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */

#include <string.h>

#include "bfs.h"
#include "flash.h"
#include "lcd.h"
#include "fonty.h"
#include "global.h"


/* ================== Datentypen und Variablendefinitionen ================= */

extern const unsigned long secaddr[19];


/* ======================= Funktionen und Programmcode ===================== */

//Public functions


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_Mount

   Return true=ok

   ----------------------------------------------------------------------------
*/
unsigned char BFS_Mount ()
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned long Offset;
    unsigned long FileHeaderAddress;
    unsigned char RecordInfo;
    unsigned long Id;
    unsigned short Len;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;
    const unsigned long BfsVersion = BFS_VERSION;


   /* ----- Programmcode ----- */

    //Format
    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        //lcd_fill (0);//Clr screen
        //draw_string (0, 70, "init file system", LCD_COLOR_B, DRAW_PUT);
        //draw_string (0, 80,"BfsSektor:", LCD_COLOR_B, DRAW_PUT);
        //draw_hexC (60,80,BfsSector, LCD_COLOR_B, DRAW_PUT);

        if (!BFS_IsSectorFormated (BfsSector))
        {
            //Unformated sector
            //Erase sector
            if (eraseSector (BFS_CHIP, BFS_FIRST_SECTOR + BfsSector) != 0)
            {
                return false;//Error
            }
            //Write sector header
            Offset = BFS_GetPhyByteAddrFromBfsSector (BfsSector);
            writeBuffer (Offset, (unsigned char*)BFS_MAGIC, 4);
            writeBuffer (Offset + 4, (unsigned char*)&BfsVersion, 4);
        }
    }

    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        glBfsDeletedBytes[BfsSector] = 0;
        glBfsOffsetToFreeRecord[BfsSector] = 0;

        FileHeaderAddress = BFS_SECTOR_HEADER_LEN + BFS_GetPhyByteAddrFromBfsSector (BfsSector);

        do
        {
            BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &Id, &Len, &FileBodyAddress, &IsFree, &IsActive);
            if (IsFree)
            {
                glBfsOffsetToFreeRecord[BfsSector] = FileHeaderAddress - BFS_GetPhyByteAddrFromBfsSector (BfsSector);
            }
            else
            {
                if (!IsActive)
                {
                    //Deleted
                    glBfsDeletedBytes[BfsSector] += (FileBodyAddress - FileHeaderAddress) + Len;
                }
                FileHeaderAddress = FileBodyAddress + Len;
            }
        }while (!IsFree);
    }

    //Init fill order
    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        glBfsFillOrder[BfsSector] = BfsSector;
    }
    //Sort fill order
    BFS_SortArray ();

    //Free sector number
    if (glBfsOffsetToFreeRecord[glBfsFillOrder[BFS_SECTORS-1]] > BFS_SECTOR_HEADER_LEN)
    {
        //No free sector
        //ASSERT (false);
        return false;//Error
    }

    return true;//OK
}
/* ----- Ende BFS_Mount ---------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_LoadFile

   Return: ReadLen

   ----------------------------------------------------------------------------
*/
unsigned short BFS_LoadFile (unsigned long Id, unsigned short MaxLen, unsigned char *Buffer)
{
   /* ----- lokale Variablen ----- */
    unsigned long FileHeaderAddress;
    unsigned char RecordInfo;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;


   /* ----- Programmcode ----- */
    if (!BFS_GetFileHeaderAddress (Id, &FileHeaderAddress))
    {
        return 0;//File not found
    }
    BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);
    memcpy (Buffer, (unsigned char*)FileBodyAddress, min (FileBodyLen, MaxLen));

    return min (FileBodyLen, MaxLen);//Error
}
/* ----- Ende BFS_LoadFile ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_CmpFile

   Return: ReadLen

   ----------------------------------------------------------------------------
*/
unsigned char BFS_CmpFile (unsigned long Id, unsigned short MaxLen, unsigned char *Buffer)
{
   /* ----- lokale Variablen ----- */
    unsigned long FileHeaderAddress;
    unsigned char RecordInfo;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;
    int x;


   /* ----- Programmcode ----- */
    if (!BFS_GetFileHeaderAddress (Id, &FileHeaderAddress))
    {
        return 0;//File not found
    }
    BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);
    x=memcmp (Buffer, (unsigned char*)FileBodyAddress, min (FileBodyLen, MaxLen));

    if (x)
        return 0;//not equal
    return 1;
}
/* ----- Ende BFS_CmpFile -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_LoadFileAddr

   Return: ReadLen

   ----------------------------------------------------------------------------
*/
void* BFS_LoadFileAddr (unsigned long Id)
{
   /* ----- lokale Variablen ----- */
    unsigned long FileHeaderAddress;
    unsigned char RecordInfo;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;


   /* ----- Programmcode ----- */
    if (!BFS_GetFileHeaderAddress (Id, &FileHeaderAddress))
    {
        return 0;//File not found
    }
    BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);

    return (void*)FileBodyAddress;

}
/* ----- Ende BFS_LoadFileAddr --------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_SaveFile

   Return true=OK

   ----------------------------------------------------------------------------
*/
unsigned char BFS_SaveFile (unsigned long Id, unsigned short Len, unsigned char *Buffer)
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned char RecordInfo;
    int i;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;
    unsigned char OverridePossible;
    unsigned char CreateNewFile;
    unsigned char Byte;
    unsigned long FileHeaderAddress;
    unsigned char HeaderSize;


   /* ----- Programmcode ----- */
    CreateNewFile = false;
    if (BFS_GetFileHeaderAddress (Id, &FileHeaderAddress))
    {
        //File is already present
        BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);
        //Same len?
        OverridePossible = false;
        if (Len == FileBodyLen)
        {
            //Is override possible (no change from 0 to 1)?
            OverridePossible = true;
            for (i=0; i<Len; i++)
            {
                if (( (unsigned char*)Buffer)[i] & (0xFF ^ readByte (FileBodyAddress+i)))
                {
                    //0->1 error
                    OverridePossible = false;
                }
            }
        }
        if (OverridePossible)
        {
            if (writeBuffer (FileBodyAddress, Buffer, Len) != 0)
            {
                return false;//Error
            }
        }
        else
        {
            //Delete existing file
            Byte = (RecordInfo & 0x3F) | (BFS_TYPE_DELETED << 6);
            writeBuffer (FileHeaderAddress, &Byte, 1);
            glBfsDeletedBytes[BFS_ByteAddressToBfsSector (FileHeaderAddress)] += (FileBodyAddress - FileHeaderAddress) + FileBodyLen;

            CreateNewFile = true;
        }
    }
    else
    {
        //File not present
        CreateNewFile = true;
    }

    while (CreateNewFile)
    {
        //Check free space
        //Start with the fullest, ignore the last empty one
        for (i=0; (i< (BFS_SECTORS-1)) && CreateNewFile; i++)
        {
            BfsSector = glBfsFillOrder[i];
            FileHeaderAddress = glBfsOffsetToFreeRecord[BfsSector];
            if ((FileHeaderAddress + BFS_MAX_RECORD_HEADER_LEN + Len) < BFS_SECTOR_SIZE)
            {
                //Enough space free
                BFS_WriteAtAddress (BFS_GetPhyByteAddrFromBfsSector (BfsSector) + FileHeaderAddress, Id, Len, Buffer, &HeaderSize);

                glBfsOffsetToFreeRecord[BfsSector] += HeaderSize + Len;
                BFS_SortArray ();

                CreateNewFile = false;
            }
        }

        if (CreateNewFile)
        {
            //No space free
            //copy only active files (not the files markes as deleted) to the empty sector and erase the source sector
            if (!BFS_MakeSpace (Len))
            {
                //Still not enough space free
                return false;//Error
            }
        }
    }

    return true;//OK
}
/* ----- Ende BFS_SaveFile ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_DeleteFile

   Return true=OK
   Return false=File not found/File was already deleted

   ----------------------------------------------------------------------------
*/
unsigned char BFS_DeleteFile (unsigned long Id)
{
   /* ----- lokale Variablen ----- */
    unsigned char RecordInfo;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;
    unsigned char Byte;
    unsigned long FileHeaderAddress;


   /* ----- Programmcode ----- */
    if (BFS_GetFileHeaderAddress (Id, &FileHeaderAddress))
    {
        //File is present
        BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);

        //Delete existing file
        Byte = (RecordInfo & 0x3F) | (BFS_TYPE_DELETED << 6);
        writeBuffer (FileHeaderAddress, &Byte, 1);

        glBfsDeletedBytes[BFS_ByteAddressToBfsSector (FileHeaderAddress)] += (FileBodyAddress - FileHeaderAddress) + FileBodyLen;
        return true;//OK
    }

    return false;//File not found/File was already deleted
}
/* ----- Ende BFS_DeleteFile ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_DeleteAllFiles

   Return true=ok

   ----------------------------------------------------------------------------
*/
unsigned char BFS_DeleteAllFiles ()
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned long Offset;
    const unsigned long BfsVersion = BFS_VERSION;


   /* ----- Programmcode ----- */

    //Format
    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        //Erase sector
        if (eraseSector (BFS_CHIP, BFS_FIRST_SECTOR + BfsSector) != 0)
        {
            return false;//Error
        }
        //Write sector header
        Offset = BFS_GetPhyByteAddrFromBfsSector (BfsSector);
        writeBuffer (Offset, (unsigned char*)BFS_MAGIC, 4);
        writeBuffer (Offset + 4, (unsigned char*)&BfsVersion, 4);

        glBfsDeletedBytes[BfsSector] = 0;
        glBfsOffsetToFreeRecord[BfsSector] = BFS_SECTOR_HEADER_LEN;
    }

    //Sort fill order
    BFS_SortArray ();

    return true;//OK
}
/* ----- Ende BFS_DeleteAllFiles ------------------------------------------- */


//Internal functions


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_FlashToFlashCopy

   Return true=ok

   ----------------------------------------------------------------------------
*/
unsigned char BFS_FlashToFlashCopy (unsigned long DstByteAddr, unsigned long SrcByteAddr, unsigned short ByteLen)
{
   /* ----- Programmcode ----- */
    writeBuffer (DstByteAddr, (unsigned char*)SrcByteAddr, ByteLen);

    return true;//OK
}
/* ----- Ende BFS_FlashToFlashCopy ----------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_MakeSpace

   Return true=ok (now it is space free for the required size)

   ----------------------------------------------------------------------------
*/
unsigned char BFS_MakeSpace (unsigned short RequiredFileBodySize)
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned long Offset;
    unsigned long DstAddress;
    unsigned long FileHeaderAddress;
    unsigned long FileBodyAddress;
    unsigned short FileBodyLen;
    unsigned char IsFree;
    unsigned char IsActive;
    unsigned long Id;
    unsigned char RecordInfo;
    unsigned char BfsSectorWithMostDelFiles;
    const unsigned long BfsVersion = BFS_VERSION;


   /* ----- Programmcode ----- */
    do
    {
        BfsSector = glBfsFillOrder[BFS_SECTORS - 2];
        FileHeaderAddress = glBfsOffsetToFreeRecord[BfsSector];
        if ((FileHeaderAddress + BFS_MAX_RECORD_HEADER_LEN + RequiredFileBodySize) < BFS_SECTOR_SIZE)
        {
            //Enough space free
            return true;//OK
        }

        //copy only active files (not the files markes as deleted)
        //to the empty sector and erase the source sector

        //Calc sector with most bytes marked as deleted
        BfsSectorWithMostDelFiles = 0;
        for (BfsSector=0; BfsSector< (BFS_SECTORS-1); BfsSector++)
        {
            if (glBfsDeletedBytes[BfsSector] > glBfsDeletedBytes[BfsSectorWithMostDelFiles])
            {
                BfsSectorWithMostDelFiles = BfsSector;
            }
        }
        if (glBfsDeletedBytes[BfsSectorWithMostDelFiles] == 0)
        {
            //No sector has files marked as deleted -> nothing to free
            return false;//Not enough space free
        }

        //Copy active files to the empty bfs sector
        BfsSector = glBfsFillOrder[BFS_SECTORS - 1];//The empty destination sector
        DstAddress = BFS_GetPhyByteAddrFromBfsSector (BfsSector) + BFS_SECTOR_HEADER_LEN;

        FileHeaderAddress = BFS_GetPhyByteAddrFromBfsSector (BfsSectorWithMostDelFiles) + BFS_SECTOR_HEADER_LEN;//Source
        do
        {
            BFS_GetFileHeaderInfo (FileHeaderAddress, &RecordInfo, &Id, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);
            if (IsActive)
            {
                //Copy the active file to the destination sector
                BFS_FlashToFlashCopy (DstAddress, FileHeaderAddress, (FileBodyAddress - FileHeaderAddress) + FileBodyLen);
                DstAddress += (FileBodyAddress - FileHeaderAddress) + FileBodyLen;

                glBfsOffsetToFreeRecord[BfsSector] += (FileBodyAddress - FileHeaderAddress) + FileBodyLen;
            }
            FileHeaderAddress = FileBodyAddress + FileBodyLen;
        }while (!IsFree);

        //Erase the source sector
        if (eraseSector (BFS_CHIP, BFS_FIRST_SECTOR + BfsSectorWithMostDelFiles) != 0)
        {
            return false;//Error
        }
        //Write sector header
        Offset = BFS_GetPhyByteAddrFromBfsSector (BfsSectorWithMostDelFiles);
        writeBuffer (Offset, (unsigned char*)BFS_MAGIC, 4);
        writeBuffer (Offset + 4, (unsigned char*)&BfsVersion, 4);

        glBfsDeletedBytes[BfsSectorWithMostDelFiles] = 0;
        glBfsOffsetToFreeRecord[BfsSectorWithMostDelFiles] = BFS_SECTOR_HEADER_LEN;

        BFS_SortArray ();
    }while (1);

    return false;//Not enough space free
}
/* ----- Ende BFS_MakeSpace ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_SortArray

   Sort the glBfsFillOrder array

   ----------------------------------------------------------------------------
*/
void BFS_SortArray ()
{
   /* ----- lokale Variablen ----- */
    unsigned char RepeatSort;
    unsigned char BfsSector;
    unsigned char Byte;


   /* ----- Programmcode ----- */
    do
    {
        RepeatSort = false;
        for (BfsSector=0; BfsSector< (BFS_SECTORS-1); BfsSector++)
        {
            if (glBfsOffsetToFreeRecord[glBfsFillOrder[BfsSector]] < glBfsOffsetToFreeRecord[glBfsFillOrder[BfsSector+1]])
            {
                //Swap
                Byte = glBfsFillOrder[BfsSector];
                glBfsFillOrder[BfsSector] = glBfsFillOrder[BfsSector+1];
                glBfsFillOrder[BfsSector+1] = Byte;
                RepeatSort = true;
            }
        }
    }while (RepeatSort);
}
/* ----- Ende BFS_SortArray ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_GetPhyByteAddrFromBfsSector
   ----------------------------------------------------------------------------
*/
unsigned long BFS_GetPhyByteAddrFromBfsSector (unsigned char BfsSector)
{

   /* ----- Programmcode ----- */
    //BFS_FIRST_SECTOR + BfsSector = PhyFlashSector
    //ASSERT ((BFS_FIRST_SECTOR + BfsSector) < FLASH_SECTORS);
    return BFS_FLASH_BASE + (secaddr[BfsSector + BFS_FIRST_SECTOR] * 2);
}
/* ----- Ende BFS_GetPhyByteAddrFromBfsSector ------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_IsSectorFormated

   Is the sector formated and is the version valid

   ----------------------------------------------------------------------------
*/
unsigned char BFS_IsSectorFormated (unsigned char BfsSector)
{
   /* ----- lokale Variablen ----- */
    unsigned long Offset;
    unsigned long Version;


   /* ----- Programmcode ----- */
    Offset = BFS_GetPhyByteAddrFromBfsSector (BfsSector);

    if (memcmp ((unsigned char*)Offset, BFS_MAGIC, 4) != 0)
    {
        return false;//Magic not found -> not formated
    }

    Version = * ((unsigned long*) (Offset + 4));
    if (Version != BFS_VERSION)
    {
        return false;//Magic found but wrong version
    }

    return true;//Formated
}
/* ----- Ende BFS_IsSectorFormated ----------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_GetFileHeaderInfo
   ----------------------------------------------------------------------------
*/
void BFS_GetFileHeaderInfo (unsigned long FileHeaderAddress, unsigned char *pRecordInfo, unsigned long *pId, unsigned short *pLen, unsigned long *pFileBodyAddress, unsigned char *pIsFree, unsigned char *pIsActive)
{
   /* ----- lokale Variablen ----- */
    unsigned char State;
    unsigned char IdByteCount;
    unsigned char LenByteCount;
    unsigned long Offset;


   /* ----- Programmcode ----- */
    Offset = FileHeaderAddress;
    *pRecordInfo = readByte (Offset++);
    State = ((*pRecordInfo) >> 6) & 0x03;
    IdByteCount = 1 + (( (*pRecordInfo) >> 1) & 0x03);
    LenByteCount = 1 + ((*pRecordInfo) & 0x01);

    if (State == BFS_TYPE_FREE)
    {
        //Free
        *pIsFree = true;
        *pIsActive = false;
        *pId = 0;
        *pLen = 0;
        *pFileBodyAddress = 0;
    }
    else
    {
        //Active or deleted
        *pIsFree = false;
        *pIsActive = (State == BFS_TYPE_ACTIVE) ? true : false;

        *pId = readByte (Offset++);
        if (IdByteCount > 1)
        {
            *pId |= readByte (Offset++) << 8;
        }
        if (IdByteCount > 2)
        {
            *pId |= readByte (Offset++) << 16;
        }
        if (IdByteCount > 3)
        {
            *pId |= readByte (Offset++) << 24;
        }

        *pLen = readByte (Offset++);
        if (LenByteCount > 1)
        {
            *pLen |= readByte (Offset++) << 8;
        }

        *pFileBodyAddress = Offset;
    }
}
/* ----- Ende BFS_GetFileHeaderInfo ---------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_GetFileHeaderAddress

   Return true = File is present, pFileHeaderAddress is valid
   Return false = File not found (e.g. deleted), pFileHeaderAddress is invalid

   ----------------------------------------------------------------------------
*/
unsigned char BFS_GetFileHeaderAddress (unsigned long Id, unsigned long *pFileHeaderAddress)
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned long PhyAddr;
    unsigned char RecordInfo;
    unsigned long TempId;
    unsigned short FileBodyLen;
    unsigned long FileBodyAddress;
    unsigned char IsFree;
    unsigned char IsActive;


   /* ----- Programmcode ----- */
    *pFileHeaderAddress = 0;

    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        PhyAddr = BFS_GetPhyByteAddrFromBfsSector (BfsSector) + BFS_SECTOR_HEADER_LEN;

        do
        {
            BFS_GetFileHeaderInfo (PhyAddr, &RecordInfo, &TempId, &FileBodyLen, &FileBodyAddress, &IsFree, &IsActive);
            if (IsActive && (TempId == Id))
            {
                //Found the correct file
                *pFileHeaderAddress = PhyAddr;
                return true;//OK
            }
            if (!IsFree)
            {
                //Skip this file
                PhyAddr = FileBodyAddress + FileBodyLen;
            }
        }while (!IsFree);
    }

    return false;//File not found
}
/* ----- Ende BFS_GetFileHeaderAddress ------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_ByteAddressToBfsSector
   ----------------------------------------------------------------------------
*/
unsigned char BFS_ByteAddressToBfsSector (unsigned long FileHeaderAddress)
{
   /* ----- lokale Variablen ----- */
    unsigned char BfsSector;
    unsigned long ByteAddress;


   /* ----- Programmcode ----- */
    for (BfsSector=0; BfsSector<BFS_SECTORS; BfsSector++)
    {
        ByteAddress = BFS_GetPhyByteAddrFromBfsSector (BfsSector);
        if ((FileHeaderAddress >= ByteAddress) && (FileHeaderAddress < (ByteAddress + BFS_SECTOR_SIZE)))
        {
            return BfsSector;
        }
    }

    //ASSERT (false);
    BfsSector = 0xFF;//Error
    return BfsSector;
}
/* ----- Ende BFS_ByteAddressToBfsSector ----------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: BFS_WriteAtAddress

   Return true=ok

   ----------------------------------------------------------------------------
*/
unsigned char BFS_WriteAtAddress (unsigned long FileHeaderAddress, unsigned long Id, unsigned short Len, unsigned char *Buffer, unsigned char *pHeaderSize)
{
   /* ----- lokale Variablen ----- */
    unsigned char RecordHeaderPos;
    unsigned char RecordInfo;
    unsigned char RecordHeader[BFS_MAX_RECORD_HEADER_LEN];
    unsigned char IdByteCount;
    unsigned char LenByteCount;


   /* ----- Programmcode ----- */

    //IdByteCount
    if (Id & 0xFF000000)
    {
        IdByteCount = BFS_ID_4BYTES;
    }
    else
    {
        if (Id & 0xFF0000)
        {
            IdByteCount = BFS_ID_3BYTES;
        }
        else
        {
            if (Id & 0xFF00)
            {
                IdByteCount = BFS_ID_2BYTES;
            }
            else
            {
                IdByteCount = BFS_ID_1BYTE;
            }
        }
    }

    //LenByteCount
    LenByteCount = (Len < 0x100) ? BFS_ONE_LEN_BYTE : BFS_TWO_LEN_BYTES;

    //RecordInfo
    RecordInfo = (BFS_TYPE_ACTIVE << 6) | (IdByteCount << 1) | LenByteCount;
    RecordHeaderPos = 0;
    RecordHeader[RecordHeaderPos++] = RecordInfo;

    //Id
    RecordHeader[RecordHeaderPos++] = Id & 0xFF;
    if (Id >= 0x100)
    {
        RecordHeader[RecordHeaderPos++] = (Id >> 8) & 0xFF;
        if (Id >= 0x10000)
        {
            RecordHeader[RecordHeaderPos++] = (Id >> 16) & 0xFF;
            if (Id >= 0x1000000)
            {
                RecordHeader[RecordHeaderPos++] = (Id >> 24) & 0xFF;
            }
        }
    }

    //Len
    RecordHeader[RecordHeaderPos++] = Len & 0xFF;
    if (Len >= 0x100)
    {
        RecordHeader[RecordHeaderPos++] = (Len >> 8) & 0xFF;
    }

    //Write record header
    writeBuffer (FileHeaderAddress, RecordHeader, RecordHeaderPos);
    //Write record data
    writeBuffer (FileHeaderAddress + RecordHeaderPos, Buffer, Len);

    *pHeaderSize = RecordHeaderPos;

    return true;//OK
}
/* ----- Ende BFS_WriteAtAddress ------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: readByte
   ----------------------------------------------------------------------------
*/
unsigned char readByte (unsigned long ByteAddr)
{
   /* ----- lokale Variablen ----- */
    unsigned char Byte;


   /* ----- Programmcode ----- */
    Byte = ((unsigned char*) ByteAddr)[0];

    return Byte;
}
/* ----- Ende readByte ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: readWord

   addr = unsigned short address

   ----------------------------------------------------------------------------
*/
unsigned short readWord (unsigned long WordAddr)
{
   /* ----- lokale Variablen ----- */
    int Offset;
    unsigned short Word;


   /* ----- Programmcode ----- */
    //ASSERT ((WordAddr * 2) >= FLASH1_BASE);
    //ASSERT ((WordAddr * 2) < (FLASH1_BASE + FLASH_SIZE));

    Offset = (WordAddr * 2) - FLASH1_BASE;

    Word = * ((unsigned short *) (WordAddr<<1));

    return Word;
}
/* ----- Ende readWord ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: writeBuffer

   Return 0=ok

   ----------------------------------------------------------------------------
*/
int writeBuffer (unsigned long ByteAddr, unsigned char *buffer, unsigned short len)
{
   /* ----- lokale Variablen ----- */
    int Offset;
    unsigned short Word;


   /* ----- Programmcode ----- */
    Offset = 0;

    if (ByteAddr & 1)
    {
        //Not word boundary
        Word = readWord (ByteAddr / 2);
        Word &= (buffer[Offset] << 8) | 0xFF;
        writeWord (ByteAddr & 0xFFFFFFFE, Word);
        Offset++;
        ByteAddr++;
        len--;
    }

    while (len >= 2)
    {
        Word = readWord (ByteAddr / 2);
        Word &= buffer[Offset] | (buffer[Offset + 1] << 8);
        writeWord (ByteAddr, Word);
        Offset += 2;
        ByteAddr += 2;
        len -= 2;
    }

    if (len == 1)
    {
        Word = readWord (ByteAddr / 2);
        Word &= 0xFF00 | buffer[Offset];
        writeWord (ByteAddr, Word);
        Offset++;
        ByteAddr++;
        len--;
    }

    return 0;//OK
}
/* ----- Ende writeBuffer -------------------------------------------------- */


/* =================================== Ende ================================ */

