/*
    sokoban.h - game
    Copyright (C) 2007  Colibri <colibri_dvb@lycos.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */

#include "../global.h"


/* ================== Datentypen und Variablendefinitionen ================= */


#ifndef SOKOBAN_H
#define SOKOBAN_H

#define SOKOBAN_MAX_UNDOS    100
#define SOKOBAN_UNDO_DIR_LEFT    0
#define SOKOBAN_UNDO_DIR_RIGHT    1
#define SOKOBAN_UNDO_DIR_UP        2
#define SOKOBAN_UNDO_DIR_DOWN    3

#define SPACE    0
#define GOAL    1
#define PUSHER    2
#define PACKET    4
#define WALL    8
#define PUSHER_ON_GOAL    (PUSHER | GOAL)
#define PACKET_ON_GOAL    (PACKET | GOAL)

#define SOKOBAN_CELLS_X    21
#define SOKOBAN_CELLS_Y    19
//Size without grid
#define SOKOBAN_CELL_SIZE    5

struct _SokobanGameInfo
{
    BYTE G[SOKOBAN_CELLS_X][SOKOBAN_CELLS_Y];
    short Level;
    int PusherX;
    int PusherY;
    WORD Moves;
    WORD UndoCount;
    BYTE UndoBuffer[SOKOBAN_MAX_UNDOS][3];
};


/* ======================== Prototyping ==================================== */

void GameSokoban (void* pData);
void GameSokoban_InitGameField (struct _SokobanGameInfo *pGI);
void GameSokoban_DrawCell (int x, int y, BYTE Object);
BOOL GameSokoban_IsSolved (BYTE G[SOKOBAN_CELLS_X][SOKOBAN_CELLS_Y]);
void GameSokoban_InitDisplayForNewGame (struct _SokobanGameInfo *pGI);
void GameSokoban_Move (struct _SokobanGameInfo *pGI);
void GameSokoban_SaveUndo (struct _SokobanGameInfo *pGI, BYTE Count, BYTE Dir);
void GameSokoban_UndoAMove (struct _SokobanGameInfo *pGI);


#endif


/* =================================== Ende ================================ */

