/*
    keyboard.c - button handling
    Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */

#include "keyboard.h"
#include "keyirq.h"
#include "lpc2220.h"
#include "timerfuncs.h"
#include "backlight.h"
#include "irq.h"


/* ================== Datentypen und Variablendefinitionen ================= */


/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: initKeys
   ----------------------------------------------------------------------------
*/
void initKeys (void)
{
   /* ----- Programmcode ----- */
    keys[0] = 0;
    keys[1] = 0;
    IOCLR2= ((1<<18)| (1<<19)| (1<<20)| (1<<21)| (1<<22)| (1<<23)| (1<<24));
    FIODIR0 &= ~ (1<<30);
    startCB (addTimerCB (keyScanner, 15));

    EXTWAKE |= 0x08;
    PINSEL1 |= (1<<29);
}
/* ----- Ende initKeys ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: waitKeyUp
   ----------------------------------------------------------------------------
*/
void waitKeyUp (void)
{
   /* ----- Programmcode ----- */

   //while (scannerState != S_IDLE) {};
   /* LeoTheLoewe: Actually this is incorrect but we use it because
                   of key hold and key repeat.
      @see isKeyPressed ()
   */
   while ((keys[0] != 0) || (keys[1] != 0)) {};
}
/* ----- Ende waitKeyUp ----------------------------------------------------- */


extern void cpu_idle ();


/*
   ----------------------------------------------------------------------------
   Funktion: waitKeyDown
   ----------------------------------------------------------------------------
*/
void waitKeyDown (void)
{
   /* ----- Programmcode ----- */
    while ((keys[0] == 0) && (keys[1] == 0))
    {
        cpu_idle ();
    };
    setBacklight (BL_AUTO);    // pwm value
}
/* ----- Ende waitKeyDown -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: isKeyPressed
   ----------------------------------------------------------------------------
*/
inline char isKeyPressed (void)
{
   /* ----- Programmcode ----- */
   return (scannerState != S_IDLE);
   //return ((keys[0] != 0) || (keys[1] != 0));
}
/* ----- Ende isKeyPressed ------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: getNumKeyValue
   ----------------------------------------------------------------------------
*/
signed char getNumKeyValue (void)
{
   /* ----- lokale Variablen ----- */
    signed char x;


   /* ----- Programmcode ----- */
    switch (keys[0])
    {
        case 0x00000002:
            x= 7;
            break;
        case 0x00000008:
            x= 4;
            break;
        case 0x00000020:
            x= 1;
            break;
        case 0x00020000:
            x= 0;
            break;
        case 0x00040000:
            x= 9;
            break;
        case 0x00080000:
            x= 8;
            break;
        case 0x00100000:
            x= 6;
            break;
        default:
            x=-1;
    }

    switch (keys[1])
    {
        case 0x00000040:
            x= 2;
            break;
        case 0x00000080:
            x= 3;
            break;
        case 0x00040000:
            x= 5;
            break;
    }
    return (x);
}
/* ----- Ende getNumKeyValue ----------------------------------------------- */


#define keynum( _X, _Y ) if (_X) return _Y


/*
   ----------------------------------------------------------------------------
   Funktion: getKeynum
   ----------------------------------------------------------------------------
*/
unsigned char getKeynum (void)
{
   /* ----- Programmcode ----- */
        keynum (KEY_A, 0);
        keynum (KEY_B, 1);
        keynum (KEY_C, 2);
        keynum (KEY_D, 3);
        keynum (KEY_Betty, 4);
        keynum (KEY_Exit, 5);
        keynum (KEY_Up, 6);
        keynum (KEY_Down, 7);
        keynum (KEY_Left, 8);
        keynum (KEY_Right, 9);
        keynum (KEY_OK, 10);
        keynum (KEY_Vplus, 11);
        keynum (KEY_Vminus, 12);
        keynum (KEY_Mute, 13);
        keynum (KEY_Pplus, 14);
        keynum (KEY_Pminus, 15);
        keynum (KEY_1, 16);
        keynum (KEY_2, 17);
        keynum (KEY_3, 18);
        keynum (KEY_4, 19);
        keynum (KEY_5, 20);
        keynum (KEY_6, 21);
        keynum (KEY_7, 22);
        keynum (KEY_8, 23);
        keynum (KEY_9, 24);
        keynum (KEY_0, 25);
        keynum (KEY_Minus, 26);
        keynum (KEY_AV, 27);
        keynum (KEY_Menu, 28);
        keynum (KEY_PiP, 29);
        keynum (KEY_AB, 30);
        keynum (KEY_16_9, 31);
        keynum (KEY_Info, 32);
        keynum (KEY_VTX1, 33);
        keynum (KEY_VTX2, 34);
        keynum (KEY_VTX3, 35);
        keynum (KEY_Blue, 36);
        keynum (KEY_Yellow, 37);
        keynum (KEY_Green, 38);
        keynum (KEY_Red, 39);
        keynum (KEY_TV, 40);
        keynum (KEY_Power, 41);
        return 42;
}
/* ----- Ende getKeynum ---------------------------------------------------- */


/* =================================== Ende ================================ */

