/*
main.c - main program routines
Copyright 2007  Ch. Klippel <ck@mamalala.net>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


#include <string.h>

#include "lpc2220.h"

#include "lcd.h"
#include "irq.h"
#include "keyboard.h"
#include "keyirq.h"
#include "lcd.h"
#include "fonty.h"
#include "sid.h"

#include "sound.h"
#include "echolot.h"
#include "initialisierung.h"
#include "demomode.h"

#include "serial.h"
#include "flash.h"
#include "rf.h"
#include "menu.h"
#include "testmenu.h"
#include "mainmenu.h"
#include "timerfuncs.h"
#define  __MAIN_C__
#include "global.h"
#include "adc.h"
#include "backlight.h"
#include "rtc.h"
#include "i2c.h"
#include "bfs.h"
#include "settingsmenu.h"
#include "controls.h"
#include "buffer.h"

#include "starlight.h"
#include "rfstarlight.h"


/* ================== Datentypen und Variablendefinitionen ================= */

// defines fuer Debug- und Traceausgaben
#define  DEBUG_MAIN  0

#define  DEMO_SEQUENCE_TIMEOUT   TIME_MSEC2TICKS(165000)  // Timer Tic ist 5 ms -> Makro rechnet um

unsigned char sysInfo;

extern volatile char timeout;

// extern ir_sfn    irSend;
// extern ir_fn    irRepeat;
// extern ir_fn    irStop;

const char hval[16] = "0123456789ABCDEF";

unsigned int oldkeys[2];
unsigned int key_state;
unsigned int reset_counter    = 0;
unsigned int init_counter     = 0;
unsigned int demo_counter     = 0;
unsigned int update_counter   = 0;

volatile unsigned long demo_timer = 0;

unsigned int temp;

extern unsigned char act_encoder, act_set;

extern unsigned char bl_val;

extern struct MENU starlightSettingsMenu;

extern unsigned char sact;

/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: setSpeed
   ----------------------------------------------------------------------------
*/
void setSpeed (unsigned char sp)
{

   /* ----- Programmcode ----- */
   disableIRQ ();
   switch ( sp )
   {
      case SPEED_60:        // turbo speed (60 mhz)
         BCFG0    = 0x100005EF;
         PLLCFG   = 0x45;        // 0100.0101    = PSEL=10=4    MSEL=00101=6 = 240mhz Fcco
         PLLFEED  = 0xAA;
         PLLFEED  = 0x55;
   
         VPBDIV   = 0x00;
   
         BCFG0    = 0x100004A0;
         BCFG2    = 0x100004A0;
         BCFG1    = 0x00000C21;
         sysInfo |= SYS_TURBO;
         break;
   
      case SPEED_30:    // normal speed (30 mhz)
      default:
         BCFG0    = 0x100005EF;
         PLLCFG   = 0x42;        // 0100.0010    = PSEL=10=4    MSEL=00010=3
         PLLFEED  = 0xAA;
         PLLFEED  = 0x55;
   
         VPBDIV   = 0x02;
   
         BCFG0    = 0x10000420;
         BCFG2    = 0x10000420;
         BCFG1    = 0x00000400;
         sysInfo &= ~SYS_TURBO;
         break;
   }
   enableIRQ ();
}
/* ----- Ende setSpeed ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: drawUhrzeit
   ----------------------------------------------------------------------------
*/
void drawUhrzeit ( void )
{

   /* ----- Programmcode ----- */
   #if FF_LARGE_IDLE_CLOCK
      printTime (6,12, (struct time_ *)&rtc.time, 1);
   #else
      printTime (86,152, (struct time_ *)&rtc.time, 0);
      printDate (0,152, (struct date_ *)&rtc.date, 0);
   #endif
}
/* ----- Ende drawUhrzeit -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: drawMainscreen
   ----------------------------------------------------------------------------
*/
void drawMainscreen (void)
{
   /* ----- Programmcode ----- */
   set_font (SMALLFONT);
   lcd_fill (0);
   draw_logo ();

   draw_block (0, 0, 99, 9, LCD_COLOR_B, DRAW_ERASE);

   drawUhrzeit (  );    // Uhrzeit ausgeben
   showBat ();          // Batteriesymbol anzeigen
   redraw_symbols ();
}
/* ----- Ende drawMainscreen ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: 

   let the LPC2200 sleep until the next IRQ
   
   This function tries to enter the sleep mode and stays
   there until the next interrupt wakes up the CPU.
   If that is not possible, the function returns immediately.
   ----------------------------------------------------------------------------
*/
void cpu_idle ()
{
   /* ----- Programmcode ----- */
   
   /* fall through if a key was pressed */
   if (ANYKEY)
      return;
   /* only idle mode instead of power down when:
   *   * backlight on
   *   * IR transmission
   *   * RF is activated
   *   * sound is playing
   *   * I2C access
   *   * SYS_NOPDOWN in sysInfo is set
   */
   if (bl_val != 0 || key_state != KEY_IDLE ||
      (RF.state > RFidle) ||
      (SID.flags & SIDenable) || sact || i2ccurrentjob || (sysInfo & SYS_NOPDOWN))
   {
      /* WARNING: no code reuse here:
      * PCON = <dynamic_variable> causes timing problems */
      EXTINT = 0x08;
      PCON = PCON_IDLE;
      PLLFEED = 0xAA;
      PLLFEED = 0x55;
   }
   else
   {
      /* Der Power Down Mode wird in der main() aktiviert sobald keine Taste gedrckt ist,
         nicht auf eine Funkbertragung gewartet wird und wenn die Beleuchtung aus ist.
         Um dies zu deaktivieren muss der folgenden Code in der main.c auskommentieren */

      refresh_symbols ();
      EXTINT = 0x08;
      PCON = PCON_PDOWN;
      PLLFEED = 0xAA;
      PLLFEED = 0x55;
   }
}   // cpu_idle
/* ----- Ende cpu_idle ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: main
   ----------------------------------------------------------------------------
*/
int main (void)
{

   /* ----- Programmcode ----- */
   sysInfo = 0;

   SCS |= 0x01;
   FIODIR0 |= ((1<<21) | (1<<11) | (1<<6) | (1<<23) | (1<<19) | (1<<17));
   FIODIR0 |= (1<<12);
   FIOSET0 |= (1<<12);
   FIOCLR0 |= (1<<4);

   setSpeed (SPEED_30);

   lcd_init (0);
   serial_init ();

   startTimerIRQ ();
   startADC ();

   initKeys ();

   initSound ();
   startSoundIRQ ();

   BFS_Mount ();

   /* Funkinterface initialisieren */
   RF_init ();
   load_RF_setting ();
   startRFIRQ ();
   RF_changestate ( RFwor );

   /* I2C Interface zum SIMChip initialisieren */
   initI2C ();
   initRTC ();
   startRtcIRQ ();

   enableIRQ ();

   init_menu ();

   initBacklight ();

   oldkeys[0] = keys[0];
   oldkeys[1] = keys[0];

   key_state = KEY_IDLE;

   set_font (BOLDFONT);

   // BFS_Mount ();
   load_setting ( );
   StarLight_Load_Settings ( );

   RfStarLightInit ( );

   playSound ( ( unsigned char* ) tts_initialisierung_data, tts_initialisierung_len );
   waitSound ();

   StarLight_Init ( );

   drawMainscreen ( );

   #if DEBUG_MAIN
      serial_puts("*** Und los gehts ***\n");
   #endif

   while ( 1 )
   {
      update_active_ind ();

      if ( ( keys[0] != oldkeys[0] ) || ( keys[1] != oldkeys[1] ) )
      {
         oldkeys[0] = keys[0];
         oldkeys[1] = keys[1];
         sysInfo |= SYS_KEY; // Da Wechsel Bearbeitungflag setzen

         if ( scannerState == S_IDLE )
         {
            // zurcksetzen des reset counter
            reset_counter  = 0;
            init_counter   = 0;
            demo_counter   = 0;
            update_counter = 0;
         }
      }

      switch ( key_state )
      {
         /* ===== KEY_IDLE ===== */
         case KEY_IDLE:

            if (sysInfo & SYS_KEY)
            {
               #if DEBUG_MAIN
                  serial_puts("Enter Key pressed\n");
               #endif

               sysInfo &= ~SYS_KEY; // Bearbeitungsflag zurcknehmen

               // Da Taste bettigt wurde Hintergrundbeleuchtung wieder einschalten
               setBacklight (BL_AUTO);

               /* Prfen ob bisher berhaupt Fernbedienung urinitialisiert wurde und aktuelle betriebsdaten besitzt */

               if ( !StarLight_Init_Status() )
               {
                  /* NEIN, Betty wurde noch nicht urinitialisiert */

                  #if DEBUG_MAIN
                     serial_puts("Betty muss noch initialisiert werden\n");
                  #endif

               }

               // Eigenes Tastaturhandling fr StarLight

               // Betty neu initialisieren bei langem Bettigen
               if (KEY_Minus)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Minus pressed\n");
                  #endif

                  init_counter++;

                  if ( init_counter == 10 )
                  {
                     // Initialisierung ausgeben
                     playSound ( ( unsigned char* ) tts_initialisierung_data, tts_initialisierung_len );
                     waitSound ();

                     // Counter zurcksetzen
                     init_counter = 0;

                     // Initialisierung auslsen und Bildschirm neu aufbauen
                     StarLight_Init ( );
                     drawMainscreen ( );
                  }
               }

               // Betty Einstellungsmen aufrufen
               else if (KEY_Betty)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("BETTY pressed\n");
                  #endif

                  menu_exec (&mainMenu);
                  drawMainscreen ();
                  while (KEY_Exit);
               }

               // ShortCuts fr programmierte Dimmerfunktionen
               else if (KEY_A || KEY_B || KEY_C || KEY_D)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("A-B-C-D pressed\n");
                  #endif
                  if ( KEY_A )      { StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_A ); }
                  else if ( KEY_B ) { StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_B ); }
                  else if ( KEY_C ) { StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_C ); }
                  else if ( KEY_D ) { StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_D ); }
                  drawMainscreen (  );
               }

               // Sound schalten
               else if (KEY_Mute)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Mute pressed\n");
                  #endif

                  StarLight_SoundSchalten (  );
               }

               // Dimmerdialoge fr Dimmer 1-8
               else if ( KEY_1 || KEY_2 || KEY_3 || KEY_4 || KEY_5 || KEY_6 || KEY_7 || KEY_8 )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("0-9 pressed\n");
                  #endif

                  // KEY's 1-8 auswerten
                  if ( KEY_1 )      { StarLight_DialogDimmer ( STARLIGHT_DIMMER_1 ); }
                  else if ( KEY_2 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_2 ); }
                  else if ( KEY_3 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_3 ); }
                  else if ( KEY_4 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_4 ); }
                  else if ( KEY_5 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_5 ); }
                  else if ( KEY_6 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_6 ); }
                  else if ( KEY_7 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_7 ); }
                  else if ( KEY_8 ) { StarLight_DialogDimmer ( STARLIGHT_DIMMER_8 ); }

                  // Hauptbildschirm wieder aufbauen
                  drawMainscreen ( );
               }

               // logpress lst Abfrage Betriebsdaten und Auswertung bertemperatur aus
               else if ( KEY_0 )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("0 pressed\n");
                  #endif

                  update_counter++;

                  if ( update_counter == 10 )
                  {
                     // Counter zurcksetzen
                     update_counter = 0;

                     // Initialisierung auslsen und Bildschirm neu aufbauen
                     StarLight_Init ( );

                     // Nun noch generell nach bertemperatur der Slaves prfen
                     StarLight_Check_Uebertemperatur(  );

                     drawMainscreen ( );
                  }
               }

               // grafischer Dimmerdialog aufrufen
               else if ( KEY_OK )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("KEY_OK pressed\n");
                  #endif

                  StarLight_DimmerGUI (  );
                  drawMainscreen (  );
               }

               // Dimmer Einstellmen aufrufen
               else if ( KEY_Menu )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Menu pressed\n");
                  #endif

                  menu_exec (&starlightSettingsMenu);
                  drawMainscreen (  );
               }

               // Credits ausgeben
               else if (KEY_Info)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Info pressed\n");
                  #endif

                  Starlight_Credits (  );
                  drawMainscreen (  );
               }

               // Demomodus bei langem Bettigen
               else if (KEY_AB)
               {
                  demo_counter++;

                  if ( demo_counter == 10 )
                  {
                     // Initialisierung ausgeben
                     playSound ( ( unsigned char* ) tts_demomode_data, tts_demomode_len );
                     waitSound ();

                     // Counter zurcksetzen
                     demo_counter = 0;

                     // Demomodus auslsen und Bildschirm neu aufbauen

                     // Prfen ob Auslsung OK war
                     if ( StarLight_Demomodus(  ) == true )
                     {
                        // JA

                        // Messagebox fr Demomodus anzeigen
                        msgbox (30, BC_nobuttons, "\n    Demomode    \n");

                        // Timer fr PopUpWindow aufziehen
                        addTimeout ( &demo_timer, DEMO_SEQUENCE_TIMEOUT );

                        while ( demo_timer  )
                        {
                           // Tue nix, nur warten
                           update_active_ind ();
                        }

                        // Timeout aus Scheduler-Queue lschen
                        removeTimeout( &demo_timer );
                        demo_timer = 0;
                     }

                     // Hauptbildschirm wieder aufbauen
                     drawMainscreen ( );

                  }
               }

               // Aufruf vordefinierte Beleuchtungsstufen fr alle Lampen
               else if ( KEY_VTX1 || KEY_VTX2 || KEY_VTX3 )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("VTX1-2-3 pressed\n");
                  #endif

                  // 33%
                  if ( KEY_VTX1 )
                  {
                     StarLight_DimmerSchalten( BETTY_MASTER_BROADCAST, FKT_STATUS_EIN, STARLIGHT_DIMMER_33 );
//                     if ( StarLight_LampenDimmen ( BETTY_MASTER_BROADCAST, STARLIGHT_DIMMER_33 ) == true ) // Dimmer einstellen
//                     {
//                        // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//                        // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//                        StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );                        // Auf jeden Fall Anlage einschalten
//                     }
                  }

                  // 66%
                  else if ( KEY_VTX2 )
                  {
                     StarLight_DimmerSchalten( BETTY_MASTER_BROADCAST, FKT_STATUS_EIN, STARLIGHT_DIMMER_66 );
//                     if ( StarLight_LampenDimmen ( BETTY_MASTER_BROADCAST, STARLIGHT_DIMMER_66 ) == true ) // Dimmer einstellen
//                     {
//                        // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//                        // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//                        StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );                        // Auf jeden Fall Anlage einschalten
//                     }
                  }

                  // 100%
                  else if ( KEY_VTX3 )
                  {
                     StarLight_DimmerSchalten( BETTY_MASTER_BROADCAST, FKT_STATUS_EIN, STARLIGHT_DIMMER_100 );
//                     if ( StarLight_LampenDimmen ( BETTY_MASTER_BROADCAST, STARLIGHT_DIMMER_100 ) == true ) // Dimmer einstellen
//                     {
//                        // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//                        // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//                        StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );                         // Auf jeden Fall Anlage einschalten
//                     }
                  }
                  // Hauptbildschirm wieder aufbauen
                  drawMainscreen ( );
               }

               // Tastenpaar fr ALLE Lampen EIN/AUS
               else if ( KEY_Yellow || KEY_Blue )
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Yellow-Blue pressed\n");
                  #endif

                  // Steuerung fr alle Lampen EIN/AUS, Dimmereinstellungen bleiben erhalten
                  if ( KEY_Yellow )
                  {
                     StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );
                  }
                  else if ( KEY_Blue )
                  {
                     StarLight_AlleLampenSchalten ( FKT_STATUS_AUS );
                  }
               }

               // Anlage AUS/AN schalten
               else if (KEY_Power)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("Power pressed\n");
                  #endif

                  StarLight_AnlageSchalten(  );
               }

               // Anzeige aller Dimmerdaten
               else if (KEY_TV)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("TV pressed\n");
                  #endif

                  StarLight_ShowDimmerData( );
                  drawMainscreen ( );
               }

               // Langer Tastendruck lst 2x Echolot und letztendlich RESET aus
               else if (KEY_AV)
               {
                  // bei Bedarf Debugausgabe schreiben
                  #if DEBUG_MAIN
                     serial_puts("AV pressed\n");
                  #endif

                  reset_counter++;

                  if ( ( reset_counter == 10 ) || ( reset_counter == 20 ) )
                  {
                      playSound ( ( unsigned char* ) echolot_data, echolot_len );
                      waitSound ();

                     #if DEBUG_MAIN
                        serial_puts("*BEEP*\n");
                     #endif

                  }
                  if ( reset_counter == 30 )
                  {
                     StarLight_Forced_Reset ( );
                  }
               }

               // Taste wurde bearbeitet -> nun State umschalten
               if ( ( keys[0] != 0) || ( keys[1] != 0 ) )
               {
                  key_state = KEY_PRESS; // Zustand geht auf KEY_PRESSED
               }

            }
            break;


         /* ===== KEY_PRESSED ===== */
         case KEY_PRESS:

            // Prfen ob Tasten immer noch bettigt sind
            if ( ( keys[0] != 0) || ( keys[1] != 0 ) )
            {
               // JA
               key_state = KEY_HOLD; // Zustand geht auf KEY_PRESSED
            }
            else
            {
               // NEIN
               key_state = KEY_RELEASE; // Zustand geht auf KEY_RELEASED
            }

            break;


         /* ===== KEY_HOLD ===== */
         case KEY_HOLD:

            // Prfen on Tasten losgelassen wurde
            if ( ( keys[0] == 0 ) && ( keys[1] == 0 ) )
            {
               key_state = KEY_RELEASE;
            }
            break;


         /* ===== KEY_RELEASE ===== */
         case KEY_RELEASE:

            key_state = KEY_IDLE;
            #if DEBUG_MAIN
               serial_puts("KEY RELEASED\n");
            #endif
            break;

      }

      cpu_idle ();

      if (timeInfo & timechanged)
      {
         drawUhrzeit(  ); // Uhrzeit ausgeben
         timeInfo &= ~timechanged;
      }

   }   // while (1)

   return 0;

}   // main
/* ----- Ende main --------------------------------------------------------- */


/* =================================== Ende ================================ */

