/*
* minor changes (c) 2007 ch.klippel
*
* (C) Copyright 2004
* DAVE Srl
* http://www.dave-tech.it
* http://www.wawnet.biz
* mailto:info@wawnet.biz
*
* (C) Copyright 2002-2004
* Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
*
* (C) Copyright 2002
* Sysgo Real-Time Solutions, GmbH <www.elinos.com>
* Marius Groeger <mgroeger@sysgo.de>
*
* (C) Copyright 2002
* Sysgo Real-Time Solutions, GmbH <www.elinos.com>
* Alex Zuepke <azu@sysgo.de>
*
* Copyright (C) 1999 2000 2001 Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/



/* =================================== Includes ============================ */

#include "lpc2220.h"
#include "serial.h"
#include "irq.h"
#include "backlight.h"
#include "buffer.h"


/* ================== Datentypen und Variablendefinitionen ================= */

extern const char hval[16];

extern void uart0_irq (void);


/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_flush_input

   flush serial input queue. returns 0 on success or negative error^
   number otherwise

   ----------------------------------------------------------------------------
*/
int serial_flush_input (void)
{
   /* ----- lokale Variablen ----- */
   volatile unsigned int tmp;


   /* ----- Programmcode ----- */
   /* keep on reading as long as the receiver is not empty */
   while (RX_DATA (U0LSR)) {
      tmp = GET_CHAR (U0RBR);
   }

   return 0;
}
/* ----- Ende serial_flush_input ------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_flush_output

   flush output queue. returns 0 on success or negative error number
   otherwise

   ----------------------------------------------------------------------------
*/
int serial_flush_output (void)
{
   /* ----- Programmcode ----- */

   /* wait until the transmitter is no longer busy */
   while (TX_READY (U0LSR)==0) {
   }

   return 0;
}
/* ----- Ende serial_flush_output ------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_setbrg
   ----------------------------------------------------------------------------
*/
void serial_setbrg (unsigned int baudrate)
{
   /* ----- Programmcode ----- */

   /* get correct divisor */
   /*for PCLK 15 Mhz */
   U0LCR = 0x83;

   switch (baudrate) {

   case 75:
      U0DLL = 5;
      U0DLM = 195;
      U0FDR = (1 << 4) + 3;
      break;

   case 150:
      U0DLL = 5;
      U0DLM = 195;
      U0FDR = (1 << 4) + 1;
      break;

   case 300:
      U0DLL = 5;
      U0DLM = 195;
      U0FDR = (1 << 4) + 0;
      break;

   case 600:
      U0DLL = 2;
      U0DLM = 78;
      U0FDR = (4 << 4) + 1;
      break;

   case 1200:
      U0DLL = 1;
      U0DLM = 39;
      U0FDR = (4 << 4) + 1;
      break;

   case 2400:
      U0DLL = 186;
      U0DLM = 0;
      U0FDR = (10 << 4) + 11;
      break;

   case 4800:
      U0DLL = 93;
      U0DLM = 0;
      U0FDR = (10 << 4) + 11;
      break;

   case 9600:
      U0DLL = 71;
      U0DLM = 0;
      U0FDR = (8 << 4) + 3;
      break;

   case 14400:
      U0DLL = 31;
      U0DLM = 0;
      U0FDR = (10 << 4) + 11;
      break;

   case 19200:
      U0DLL = 38;
      U0DLM = 0;
      U0FDR = (14 << 4) + 4;
      break;

   case 28800:
      U0DLL = 24;
      U0DLM = 0;
      U0FDR = (14 << 4) + 5;
      break;

   case 38400:
      U0DLL = 19;
      U0DLM = 0;
      U0FDR = (14 << 4) + 4;
      break;

   case 57600:
      U0DLL = 12;
      U0DLM = 0;
      U0FDR = (14 << 4) + 5;
      break;

   case 115200:
      U0DLL = 6;
      U0DLM = 0;
      U0FDR = (14 << 4) + 5;
      break;

   case 230400:
      U0DLL = 3;
      U0DLM = 0;
      U0FDR = (14 << 4) + 5;
      break;
   }
   U0LCR = 0x03;

   serial_flush_output ();
   serial_flush_input ();
}
/* ----- Ende serial_setbrg ------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_init

   Initialise the serial port with the given baudrate. The settings
   are always 8 data bits, no parity, 1 stop bit, no start bits.

   ----------------------------------------------------------------------------
*/
int serial_init (void)
{
   /* ----- Programmcode ----- */

   //    PINSEL0=0x05;            // pin select -> tx, rx
   serial_setbrg (defaultbrg);
   //    serial_setbrg (57600);
   U0FCR = 0x07;            // enable fifo
   uart0_incbuffer = 0;
   uart0_outcbuffer = 0;

   // enable IRQ5 for UART0
   VICVectAddr5 = (unsigned long)& (uart0_irq);
   VICVectCntl5 = VIC_SLOT_EN | INT_SRC_UART0;
   VICIntEnable = INT_UART0;
   U0IER = IER_RBR | IER_RXLS;
   return (0);
}
/* ----- Ende serial_init -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_putc

   Output a single character to the serial port.

   ----------------------------------------------------------------------------
*/

void serial_putc (const char c)
{
   /* ----- Programmcode ----- */

   // proper CRLF transmission
   if (c == '\n')
      serial_putc ('\r');

   /* wait for room in the transmit FIFO */
   while (TX_READY (U0LSR)==0);

   PUT_CHAR (U0THR,c);
}
/* ----- Ende serial_putc -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_putbyte

   Wait until FIFO is ready and send one single byte to the serial port.

   ----------------------------------------------------------------------------
*/
void serial_putbyte (const char c)
{
   /* ----- Programmcode ----- */

   /* wait for room in the transmit FIFO */
   while (TX_READY (U0LSR)==0);

   PUT_CHAR (U0THR,c);
}
/* ----- Ende serial_putbyte ----------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_tstc

   Returns 1 if a byte has been received by the UART otherwise 0

   ----------------------------------------------------------------------------
*/
int serial_tstc (void)
{
   /* ----- Programmcode ----- */
   return (RX_DATA (U0LSR));
}
/* ----- Ende serial_tstc -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_getc

   Read a single byte from the serial port and returns it.

   ----------------------------------------------------------------------------
*/
int serial_getc (void)
{
   /* ----- lokale Variablen ----- */
   int rv;


   /* ----- Programmcode ----- */
   for (;;) {
      rv = serial_tstc ();

      if (rv > 0)
         return (GET_CHAR (U0RBR));
   }
}
/* ----- Ende serial_getc ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_puts

   Send a string to the serial port.

   ----------------------------------------------------------------------------
*/
void serial_puts (const char *s)
{
   /* ----- Programmcode ----- */
   while (*s != 0)
   {
      serial_putc (*s++);
   }
}
/* ----- Ende serial_puts -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: putHexC

   Send a byte as hexadecimal characters to the serial port.

   ----------------------------------------------------------------------------
*/
void putHexC (const unsigned char v)
{
   /* ----- Programmcode ----- */
   serial_putc (hval[v>>4]);
   serial_putc (hval[v & 0x0F]);
}
/* ----- Ende putHexC ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: putHexS

   Send a short as hexadecimal characters to the serial port.

   ----------------------------------------------------------------------------
*/
void putHexS (const unsigned short v)
{
   /* ----- Programmcode ----- */
   putHexC (v >> 8);
   putHexC (v & 0xFF);
}
/* ----- Ende putHexS ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: putHexW

   Send a long as hexadecimal characters to the serial port.

   ----------------------------------------------------------------------------
*/
void putHexW (const unsigned long v)
{
   /* ----- Programmcode ----- */
   putHexS (v >> 16);
   putHexS (v & 0xFFFF);
}
/* ----- Ende putHexW ------------------------------------------------------ */


/*
   ----------------------------------------------------------------------------
   Funktion: serial_interactive

   Receive a character and performe some activities.

   ----------------------------------------------------------------------------
*/
void serial_interactive (void)
{
   /* ----- lokale Variablen ----- */
   int ch = serial_getc ();


   /* ----- Programmcode ----- */
   switch (ch)
   {
      case '.':
         serial_puts ("Betty Boop ("__DATE__ " " __TIME__ ")\n");
      break;
      case '0':
         setBacklight (0x00);    // pwm value
      break;
      case '1':
         setBacklight (0x1F);    // pwm value
      break;
      case '2':
         setBacklight (0x3F);    // pwm value
      break;
      default:
         serial_putc (ch);
   }
}
/* ----- Ende serial_interactive ------------------------------------------- */

/* =================================== Ende ================================ */
