/*
    serial.h - serial port functions
    Copyright (C) 2007  Ch. Klippel <ck@mamalala.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


/* ================== Datentypen und Variablendefinitionen ================= */

#ifndef BOOP_SERIAL_H
#define BOOP_SERIAL_H

#define    USRRxData          (1 << 0)
#define    USRTxHoldEmpty     (1 << 6)

#define IER_RBR     1
#define IER_THRE    2
#define IER_RXLS    4

#define IIR_RLS     0x06
#define IIR_RDA     0x04
#define IIR_CTI     0x0c
#define IIR_THRE    0x02

#define GET_CHAR(p)     p
#define PUT_CHAR(p,c)   (p= (unsigned ) (c))
#define TX_READY(s)     ((s) & USRTxHoldEmpty)
#define RX_DATA(s)      ((s) & USRRxData)

#define defaultbrg    115200

struct cBuffer_* uart0_incbuffer;
struct cBuffer_* uart0_outcbuffer;


/* ======================== Prototyping ==================================== */


/** flush serial input queue. returns 0 on success or negative error
* number otherwise
*/
int  serial_flush_input (void);
/** flush output queue. returns 0 on success or negative error number
* otherwise
*/
int  serial_flush_output (void);
void serial_setbrg (unsigned int baudrate);
/** Initialise the serial port with the given baudrate. The settings
* are always 8 data bits, no parity, 1 stop bit, no start bits.
*/
int  serial_init (void);
/** Output a single character to the serial port.
*/
void serial_putc (const char c);
/** Wait until FIFO is ready and send one single byte to the serial port.
*/
void serial_putbyte (const char c);
/** Returns 1 if a byte has been received by the UART
* otherwise 0
*/
int  serial_tstc (void);
/** Read a single byte from the serial port and returns it.
*/
int  serial_getc (void);
/** Send a string to the serial port.
*/
void serial_puts (const char *s);

/** Send a byte as hexadecimal characters to the serial port.
*/
void putHexC (const unsigned char v);
/** Send a short as hexadecimal characters to the serial port.
*/
void putHexS (const unsigned short v);
/** Send a long as hexadecimal characters to the serial port.
*/
void putHexW (const unsigned long v);

/** Receive a character and performe some activities.
*/
void serial_interactive (void);

#endif /* BOOP_SERIAL_H */

/* =================================== Ende ================================ */

