/**********************************************************************
@file       starlight.c

@brief      Beinhaltet kompletten Projekt-Code fr StarLight

-----------------------------------------------------------------------
@author  2010 Markus Fulde
-----------------------------------------------------------------------
Following compile flags are usable:


-----------------------------------------------------------------------
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------
@History

1.00 17092010    MaFu    Ersterstellung

@end
***********************************************************************/

/* =================================== Includes ============================ */

// Include der Standard und Projektbibliotheken
#include <string.h>   // for memset
#include "global.h"
#include "timerfuncs.h"
#include "lpc2220.h"
#include "menu.h"
#include "controls.h"
#include "lcd.h"
#include "fonty.h"
#include "keyboard.h"
#include "backlight.h"
#include "Rfstarlight.h"
#include "starlight.h"
#include "serial.h"
#include "mainmenu.h"
#include "bfs.h"

// Include der Icons
#include "iconLampe_aus_h.h"
#include "iconLampe_ein_h.h"

#include "iconLampe_aus_v.h"
#include "iconLampe_ein_v.h"

#include "iconLicht_an.h"
#include "iconLicht_aus.h"

#include "iconBilderlampe.h"


/* ================== Datentypen und Variablendefinitionen ================= */

// DEBUG-Steuerung
#define STARLIGHT_INIT_DEBUG     0  // Debug alles was mit INIT und OPEN zu tun hat
#define CRC_DEBUG                0  // Debug alles fr CRC-Berechnung auf RS232
#define STARLIGHT_COTROL_DEBUG   0  // Debug aller Funktionen aus dem bereich Command & Controll

// Timeout festlegen
#define  MENU_SEQUENCE_TIMEOUT   TIME_MSEC2TICKS(20000)  // Timer Tic ist 5 ms -> Makro rechnet um

// berlagerung eigenes atoi
extern int atoi(const char *s);     // due to problems while include ctype.h

// Datendefinition der gesamten RF-Datenstrukturen
extern RfStarLightData_t   rfStarLightData;

// Datendefinition StarLight-Men
// extern struct MENU starlightSettingsMenu;

// Globale Variable fr bertemperaturschwelle und zugehriges Flag
volatile unsigned char  starlightUebertemperatur      = 0;
volatile BOOL           starlightUebertemperatur_flag = false;

// Globale Variable zum automatischen Verlassen des Dimmer-Einstellmen
volatile unsigned long menu_timer         = 0;

// Globale Variable fr grafische Bedienung der Dimmer laden
volatile unsigned char dimmer_index_GUI   = 1;

// Definition der persistenten Daten fr die Einstellungen von ShortCuts A-D
starlightSettings_t starlightSetting;

// Datendefiniton fr StarLight-Daten
starlightData_t starlightData;

// Datendefinition der StarLight-Settings Meneintrge
struct MENU_ENTRY starlightSettingsMenuEntries [] =
{
   { Starlight_ShortCut_A, 0, FUNCENTRY, "Funktion A", "Einstellung ShortCut A" },
   { Starlight_ShortCut_B, 0, FUNCENTRY, "Funktion B", "Einstellung ShortCut B" },
   { Starlight_ShortCut_C, 0, FUNCENTRY, "Funktion C", "Einstellung ShortCut C" },
   { Starlight_ShortCut_D, 0, FUNCENTRY, "Funktion D", "Einstellung ShortCut D" }
};

struct MENU starlightSettingsMenu =
{
    NUM_OF_ELEMENTS (starlightSettingsMenuEntries), (MENU_ENTRY*)&starlightSettingsMenuEntries, "StarLight HotKeys"
};


/* ======================= Funktionen und Programmcode ===================== */

/* ----------------------------------------------------------------------------
Funktion: starlight_credits

Funktion zur Ausgabe des Titelbildschirms

bergabeparameter:   keiner

Rckgabeparameter:   keiner
---------------------------------------------------------------------------- */
void Starlight_Credits ( void )
{
    lcd_fill( 0 );
    set_font (BOLDFONT);
    draw_string (0,0,"StarLight V1.00", LCD_COLOR_B, DRAW_PUT);
    set_font (SMALLFONT);
    draw_string (0,14,"Fernsteuerbares Multi-\n"
                      "Slave-System fuer eine\n"
                      "mehrkanalige Beleuch-\n"
                      "tungsanlage.", LCD_COLOR_B, DRAW_PUT);

    draw_string (0,100,"Danke an all die Entwickler\n"
                       "des BettyHacks Forums!", LCD_COLOR_B, DRAW_PUT);

    set_font (SMALLFONT);
    draw_string (8,140,"www.bettyhacks.com", LCD_COLOR_B, DRAW_PUT);
    draw_string (1,150,"www.kleinmaeusiges.com", LCD_COLOR_B, DRAW_PUT);


    waitKeyUpDown ();

}


/* ----------------------------------------------------------------------------
Funktion: CRC16

Funktione berechnet CRC16 Checksumme fr Kommunikation mit Master. Achtung,
Master-Implementierung erfolgt in AVR-BASCOM. CRC16 Routine ist speziell auf
BASCOM CRC16 abgestimmt!

bergabeparameter:   buffer[] = Zeiger auf Byte-Array
                     length   = Lnge ber die CRC16 gerechnet werden soll

Rckgabeparameter:   CRC16 Checksumme gemBASCOM-Implementierung
---------------------------------------------------------------------------- */
unsigned int CRC16(unsigned char buffer[ RF_TX_MESSAGE_LENGTH ], unsigned char length)
{

   /* ----- lokale Variablen ----- */
   unsigned long crc1_temp = 0; // init CRC
   unsigned long temp_long;
   unsigned char index_byte, index_bit;
   unsigned char crc_msb, crc_lsb;

   #if CRC_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */
   #if CRC_DEBUG
      sprintf( trace_string, "Checksummenberechnung mit Laenge=%d : ", length ); serial_puts( trace_string );
      for (index_byte = 0; index_byte < length; index_byte++)
      {
         sprintf( trace_string, "%X ", buffer[ index_byte ] ); serial_puts( trace_string );
      }
      serial_puts( "\n" );
   #endif

   // Schleife ber alle Bytes in Array
   for (index_byte = 0; index_byte < length; index_byte++)
   {
      crc_msb = crc1_temp / 256;
      crc_lsb = crc1_temp & 255;
      crc_msb = crc_msb ^ buffer[ index_byte ];
      crc1_temp = ( crc_msb * 256 ) + crc_lsb;

      // Schleife ber alle Bits in Bytes
      for (index_bit = 0; index_bit <= 7; index_bit++ )
      {
         temp_long = crc1_temp;
         temp_long = temp_long << 1;    // Bits um eine Position nach links schieben
         crc1_temp = temp_long & 0x0000FFFF;
         if ( temp_long > 0x0000FFFF )
         {
            // True
            crc1_temp = crc1_temp ^ 0x1021;
         }
      } // Schleife ber Bits
   } // Schleife ber Bytes

   #if CRC_DEBUG
      sprintf( trace_string, "CheckSumme : %x \n", (unsigned int) crc1_temp ); serial_puts( trace_string );
   #endif

   return crc1_temp; // CRC Rckgabewert
}
/* ----- Ende CRC16 -------------------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: FormatedDez

Funktion formatiert eine Dezimalzahl in einen String und fllt vorangestellte
Stellen die nicht benutzt werden mit NULL auf.

bergabeparameter:   string_ptr  = Pointer auf String in den konvertiert wird
                     zahl        = Zahl die konvertiert werden soll
                     stellen     = Stellen die konvertiert werden

Rckgabeparameter:   keiner (es wird auf String-Pointer gearbeitet)
---------------------------------------------------------------------------- */
void FormatedDez ( char *string_ptr, unsigned int zahl, unsigned char stellen )
{
   /* ----- lokale Variablen ----- */
   unsigned char len;
   unsigned char index;
   char temp_string[255];


   /* ----- Programmcode ----- */
   sprintf( temp_string, "%d" , zahl );
   len = strlen ( temp_string );
   for ( index = 0 ; index < stellen ; index++ )
   {
      string_ptr[ index ] = '0';
   }
   string_ptr[ stellen ] = '\0';
   index = stellen - len;
   strcpy( &string_ptr[ index ], temp_string );
}
/* ----- Ende FormatedDez -------------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Init

Funktion initialisiert Daten und Datenstrukturen fr Projekt

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
---------------------------------------------------------------------------- */
BOOL StarLight_Init ( void )
{
   /* ----- lokale Variablen ----- */
   unsigned int   return_value = true;
   unsigned char  result_value;
   unsigned char  index;
   char           temp_string[ 64 ];

   #if STARLIGHT_INIT_DEBUG
      char        trace_string[ 255 ];
   #endif


   /* ----- Programmcode ----- */

   /* Urinitialisierungs-Flag setzen */
   starlightData.initialized_flag = false;

   #if STARLIGHT_INIT_DEBUG
      serial_puts( "Enter StarLight_Init\n" );
      sprintf( trace_string, "Initializied-Flag = %x \n", starlightData.initialized_flag ); serial_puts( trace_string );
   #endif

   /* Zunchst Datenstruktur initialisieren */
   for ( index = 0; index < 8; index++ )
   {
      starlightData.dimmer[ index ].status     = 0x00;
      starlightData.dimmer[ index ].dimmwert   = 0x01;
      starlightData.dimmer[ index ].temperatur = 0x00;
      starlightData.dimmer[ index ].sound      = 0x00;
   }

   starlightData.sound_status = FKT_STATUS_AUS;

   // Font fr PopUps einstellen
   set_font (SMALLFONT);


   // Fr die Initialisierung werden zunchst die Betriebsdaten der 8 Dimmer eingelesen
   for ( index = 0; index < 8; index++ )
   {

      #if STARLIGHT_INIT_DEBUG
         sprintf( trace_string, "Sende Statusanforderung Dimmer %d\n", index+1 ); serial_puts( trace_string );
      #endif

      // Fr initialisierung PopUp auf dem Display ausgeben
      sprintf( temp_string, "\n  Initialisiere Betty  \n        Dimmer %d\n", ( index + 1 ) );
      msgbox (45, BC_nobuttons , temp_string);

      // Kommunikation mit MASTER
      result_value = RfStarLightStatusabfrage ( index + 1 );

      // Wenn Kommunikation OK dann Daten bernehmen
      if ( result_value == true )
      {
         // OK

         #if STARLIGHT_DEBUG
            sprintf( trace_string, "Verarbeite Status Dimmer %d\n", ( index + 1 ) ); serial_puts( trace_string );
         #endif

         // Prfen ob ACK Botschaft empfangen da ggf. Lampenanlage OFF
         if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
         {
            // JA - Nun noch prfen ob erwartetes Statusbyte auf 0x00
            if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
            {
               // JA

               // Fehlerbehandlung
               msgbox (45, BC_OKOnly | BC_DefaultButton1, "StarLight_Init:\nLichtanlage zeigt OFF\n");
               return_value = false;
               break; // Schleife abbrechen da Fehler

            } // FKT_STATUS_AUS

            // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
            starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
            starlightUebertemperatur_flag = true;

         } // ACKNOWLEDGE

         // Datenstrukturen auslesen und Daten in Strukt eintragen zur weiteren Verarbeitung
         starlightData.dimmer[ index ].status     = rfStarLightData.RxData.RxData.dimmer;
         starlightData.dimmer[ index ].dimmwert   = rfStarLightData.RxData.RxData.dimmwert;
         starlightData.dimmer[ index ].temperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightData.dimmer[ index ].sound      = rfStarLightData.RxData.RxData.sound;
      }
      else
      {
         // Fehlerbehandlung
         msgbox (45, BC_OKOnly | BC_DefaultButton1, "StarLight_Init:\nRF Kommunikationsfehler\nbei Initialisierung!\n");
         return_value = false;
         break; // Schleife abbrechen da Fehler
      }

      // Auf jeden Fall Empfangsbuffer leeren
      RfStarLightPurgeBuffer( );

   }

   // Wenn Kommunikation OK dann noch Status fr Sound in globale Variable bernehmen und Urinit setzen
   if ( return_value == true )
   {
      // Anmerkung: Der Soundstatus sollte in der aktuellen Implementierung fr alle
      // Dimmer gleich sein. Damit ist es egal von wo Daten bezogen werden.
      starlightData.sound_status = starlightData.dimmer[ 0 ].sound;

      /* Urinitialisierungs-Flag setzen */
      starlightData.initialized_flag = true;  // Nun ist Betty Urinitialisiert
   }

   return return_value;
}
/* ----- Ende StarLight_Init ----------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Init_Status

Funktion liefert Initialisierungs-Status der Daten und Datenstrukturen fr Projekt

bergabeparameter:      keiner

Rckgabeparameter:      TRUE  = Initialisierung erfolgt
                        FASLE = Initialisierung nocht nicht erfolgt
---------------------------------------------------------------------------- */
BOOL StarLight_Init_Status( void )
{
   return starlightData.initialized_flag;
}
/* ----- Ende StarLight_Init_Status ---------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Load_Settings

Funktion initialisiert die Einstellungen fr die Tasten A-D und lies die Daten
aus dem BFS aus.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Load_Settings( void )
{
   /* ----- lokale Variablen ----- */
   unsigned char  index_1;
   unsigned char  index_2;

   unsigned char  init_flag = false;


   /* ----- Programmcode ----- */

   // prfen ob Datenflash noch nich initialisiert wurde
   if ( BFS_LoadFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting ) != sizeof (struct starlightSettings_s) )
   {
      init_flag = true;
   }

   // prfen ib Datenversion berhaupt stimmt
   if ( starlightSetting.data_version != STARLIGHT_AKT_DATA_DEFINITION )
   {
      init_flag = true;
   }

   // Ggf. Flash neu initialisieren
   if ( init_flag == true )
   {
      // Daten urinitialisieren
      starlightSetting.data_version = STARLIGHT_AKT_DATA_DEFINITION;
      for ( index_1 = 0; index_1 < 4; index_1++ )
      {
         for ( index_2 = 0; index_2 < 8; index_2++ )
         {
            starlightSetting.data[ index_1 ][ index_2 ].dimmerstatus = STARLIGHT_FUNKTION_AUS;
            starlightSetting.data[ index_1 ][ index_2 ].dimmwert     = STARLIGHT_DIMMER_DATA_MIN;
         }
      }

      // Daten speichern
      BFS_SaveFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting );

      // Fehlerbehandlung
      msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lade A-D Default  \n");
      drawMainscreen ( );
   }

}
/* ----- Ende StarLight_Load_Settings -------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Save_Settings_A

Funktion speichert die Einstellungen fr die Funktionstasten

bergabeparameter:      Parameter der Formularverwaltung

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Save_Settings_A ( FORM* form, CONTROL* control )
{

   /* ----- Programmcode ----- */

   // Daten fr Key A rckbertragen
   starlightSetting.data[ 0 ][ 0 ].dimmerstatus = ((NUMBOX*)(form->controls[2]))->value;
   starlightSetting.data[ 0 ][ 0 ].dimmwert     = ((NUMBOX*)(form->controls[4]))->value;

   starlightSetting.data[ 0 ][ 1 ].dimmerstatus = ((NUMBOX*)(form->controls[7]))->value;
   starlightSetting.data[ 0 ][ 1 ].dimmwert     = ((NUMBOX*)(form->controls[9]))->value;

   starlightSetting.data[ 0 ][ 2 ].dimmerstatus = ((NUMBOX*)(form->controls[12]))->value;
   starlightSetting.data[ 0 ][ 2 ].dimmwert     = ((NUMBOX*)(form->controls[14]))->value;

   starlightSetting.data[ 0 ][ 3 ].dimmerstatus = ((NUMBOX*)(form->controls[17]))->value;
   starlightSetting.data[ 0 ][ 3 ].dimmwert     = ((NUMBOX*)(form->controls[19]))->value;

   starlightSetting.data[ 0 ][ 4 ].dimmerstatus = ((NUMBOX*)(form->controls[22]))->value;
   starlightSetting.data[ 0 ][ 4 ].dimmwert     = ((NUMBOX*)(form->controls[24]))->value;

   starlightSetting.data[ 0 ][ 5 ].dimmerstatus = ((NUMBOX*)(form->controls[27]))->value;
   starlightSetting.data[ 0 ][ 5 ].dimmwert     = ((NUMBOX*)(form->controls[29]))->value;

   starlightSetting.data[ 0 ][ 6 ].dimmerstatus = ((NUMBOX*)(form->controls[32]))->value;
   starlightSetting.data[ 0 ][ 6 ].dimmwert     = ((NUMBOX*)(form->controls[34]))->value;

   starlightSetting.data[ 0 ][ 7 ].dimmerstatus = ((NUMBOX*)(form->controls[37]))->value;
   starlightSetting.data[ 0 ][ 7 ].dimmwert     = ((NUMBOX*)(form->controls[39]))->value;

   // Daten speichern
   BFS_SaveFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting );

   form->flags |= BC_Form_exit;

}
/* ----- Ende StarLight_Save_Settings_A ------------------------------------ */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Save_Settings_B

Funktion speichert die Einstellungen fr die Funktionstasten

bergabeparameter:      Parameter der Formularverwaltung

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Save_Settings_B ( FORM* form, CONTROL* control )
{

   /* ----- Programmcode ----- */

   // Daten fr Key B rckbertragen
   starlightSetting.data[ 1 ][ 0 ].dimmerstatus = ((NUMBOX*)(form->controls[2]))->value;
   starlightSetting.data[ 1 ][ 0 ].dimmwert     = ((NUMBOX*)(form->controls[4]))->value;

   starlightSetting.data[ 1 ][ 1 ].dimmerstatus = ((NUMBOX*)(form->controls[7]))->value;
   starlightSetting.data[ 1 ][ 1 ].dimmwert     = ((NUMBOX*)(form->controls[9]))->value;

   starlightSetting.data[ 1 ][ 2 ].dimmerstatus = ((NUMBOX*)(form->controls[12]))->value;
   starlightSetting.data[ 1 ][ 2 ].dimmwert     = ((NUMBOX*)(form->controls[14]))->value;

   starlightSetting.data[ 1 ][ 3 ].dimmerstatus = ((NUMBOX*)(form->controls[17]))->value;
   starlightSetting.data[ 1 ][ 3 ].dimmwert     = ((NUMBOX*)(form->controls[19]))->value;

   starlightSetting.data[ 1 ][ 4 ].dimmerstatus = ((NUMBOX*)(form->controls[22]))->value;
   starlightSetting.data[ 1 ][ 4 ].dimmwert     = ((NUMBOX*)(form->controls[24]))->value;

   starlightSetting.data[ 1 ][ 5 ].dimmerstatus = ((NUMBOX*)(form->controls[27]))->value;
   starlightSetting.data[ 1 ][ 5 ].dimmwert     = ((NUMBOX*)(form->controls[29]))->value;

   starlightSetting.data[ 1 ][ 6 ].dimmerstatus = ((NUMBOX*)(form->controls[32]))->value;
   starlightSetting.data[ 1 ][ 6 ].dimmwert     = ((NUMBOX*)(form->controls[34]))->value;

   starlightSetting.data[ 1 ][ 7 ].dimmerstatus = ((NUMBOX*)(form->controls[37]))->value;
   starlightSetting.data[ 1 ][ 7 ].dimmwert     = ((NUMBOX*)(form->controls[39]))->value;

   // Daten speichern
   BFS_SaveFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting );

   form->flags |= BC_Form_exit;

}
/* ----- Ende StarLight_Save_Settings_B ------------------------------------ */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Save_Settings_C

Funktion speichert die Einstellungen fr die Funktionstasten

bergabeparameter:      Parameter der Formularverwaltung

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Save_Settings_C ( FORM* form, CONTROL* control )
{

   /* ----- Programmcode ----- */

   // Daten fr Key C rckbertragen
   starlightSetting.data[ 2 ][ 0 ].dimmerstatus = ((NUMBOX*)(form->controls[2]))->value;
   starlightSetting.data[ 2 ][ 0 ].dimmwert     = ((NUMBOX*)(form->controls[4]))->value;

   starlightSetting.data[ 2 ][ 1 ].dimmerstatus = ((NUMBOX*)(form->controls[7]))->value;
   starlightSetting.data[ 2 ][ 1 ].dimmwert     = ((NUMBOX*)(form->controls[9]))->value;

   starlightSetting.data[ 2 ][ 2 ].dimmerstatus = ((NUMBOX*)(form->controls[12]))->value;
   starlightSetting.data[ 2 ][ 2 ].dimmwert     = ((NUMBOX*)(form->controls[14]))->value;

   starlightSetting.data[ 2 ][ 3 ].dimmerstatus = ((NUMBOX*)(form->controls[17]))->value;
   starlightSetting.data[ 2 ][ 3 ].dimmwert     = ((NUMBOX*)(form->controls[19]))->value;

   starlightSetting.data[ 2 ][ 4 ].dimmerstatus = ((NUMBOX*)(form->controls[22]))->value;
   starlightSetting.data[ 2 ][ 4 ].dimmwert     = ((NUMBOX*)(form->controls[24]))->value;

   starlightSetting.data[ 2 ][ 5 ].dimmerstatus = ((NUMBOX*)(form->controls[27]))->value;
   starlightSetting.data[ 2 ][ 5 ].dimmwert     = ((NUMBOX*)(form->controls[29]))->value;

   starlightSetting.data[ 2 ][ 6 ].dimmerstatus = ((NUMBOX*)(form->controls[32]))->value;
   starlightSetting.data[ 2 ][ 6 ].dimmwert     = ((NUMBOX*)(form->controls[34]))->value;

   starlightSetting.data[ 2 ][ 7 ].dimmerstatus = ((NUMBOX*)(form->controls[37]))->value;
   starlightSetting.data[ 2 ][ 7 ].dimmwert     = ((NUMBOX*)(form->controls[39]))->value;

   // Daten speichern
   BFS_SaveFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting );

   form->flags |= BC_Form_exit;

}
/* ----- Ende StarLight_Save_Settings_C ------------------------------------ */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Save_Settings_D

Funktion speichert die Einstellungen fr die Funktionstasten

bergabeparameter:      Parameter der Formularverwaltung

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Save_Settings_D ( FORM* form, CONTROL* control )
{

   /* ----- Programmcode ----- */

   // Daten fr Key D rckbertragen
   starlightSetting.data[ 3 ][ 0 ].dimmerstatus = ((NUMBOX*)(form->controls[2]))->value;
   starlightSetting.data[ 3 ][ 0 ].dimmwert     = ((NUMBOX*)(form->controls[4]))->value;

   starlightSetting.data[ 3 ][ 1 ].dimmerstatus = ((NUMBOX*)(form->controls[7]))->value;
   starlightSetting.data[ 3 ][ 1 ].dimmwert     = ((NUMBOX*)(form->controls[9]))->value;

   starlightSetting.data[ 3 ][ 2 ].dimmerstatus = ((NUMBOX*)(form->controls[12]))->value;
   starlightSetting.data[ 3 ][ 2 ].dimmwert     = ((NUMBOX*)(form->controls[14]))->value;

   starlightSetting.data[ 3 ][ 3 ].dimmerstatus = ((NUMBOX*)(form->controls[17]))->value;
   starlightSetting.data[ 3 ][ 3 ].dimmwert     = ((NUMBOX*)(form->controls[19]))->value;

   starlightSetting.data[ 3 ][ 4 ].dimmerstatus = ((NUMBOX*)(form->controls[22]))->value;
   starlightSetting.data[ 3 ][ 4 ].dimmwert     = ((NUMBOX*)(form->controls[24]))->value;

   starlightSetting.data[ 3 ][ 5 ].dimmerstatus = ((NUMBOX*)(form->controls[27]))->value;
   starlightSetting.data[ 3 ][ 5 ].dimmwert     = ((NUMBOX*)(form->controls[29]))->value;

   starlightSetting.data[ 3 ][ 6 ].dimmerstatus = ((NUMBOX*)(form->controls[32]))->value;
   starlightSetting.data[ 3 ][ 6 ].dimmwert     = ((NUMBOX*)(form->controls[34]))->value;

   starlightSetting.data[ 3 ][ 7 ].dimmerstatus = ((NUMBOX*)(form->controls[37]))->value;
   starlightSetting.data[ 3 ][ 7 ].dimmwert     = ((NUMBOX*)(form->controls[39]))->value;

   // Daten speichern
   BFS_SaveFile ( BFS_ID_StarlightSettings, sizeof (struct starlightSettings_s), (unsigned char*) &starlightSetting );

   form->flags |= BC_Form_exit;

}
/* ----- Ende StarLight_Save_Settings_D ------------------------------------ */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Forced_Reset

Funktion lst ber WD einen SW-Reset aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Forced_Reset ( void )
{

   /* ----- Programmcode ----- */

   /* Watchdog */

   // Watchdog timer constant register
   WDTC = 0x000000FF;

   // Watchdog mode register
   /*
      WDEN WDRESET Operation
      -------------------------------------------------------------
      0    X       Debug/Operate without the Watchdog running
      1    0       Debug with the Watchdog interrupt but no WDRESET
      1    1       Operate with the Watchdog interrupt and WDRESET
   */

   WDMOD = 0x03;

   // Feed-Sequenz zum aktivieren des Watchdog ber Watchdog feed sequence register
   // Feed value should be 0xAA followed by 0x55
   WDFEED = 0xAA;
   WDFEED = 0x55;

   // Jetzt ist WD aktiv - warten auf RESET

   /* When the Watchdog counter underflows, the program counter will start
      from 0x00000000 as in the case of external reset. The Watchdog time-out
      flag (WDTOF) can be examined to determine if the Watchdog has caused
      the reset condition. The WDTOF flag must be cleared by software. */

   while( 1 );
}
/* ----- Ende StarLight_Forced_Reset --------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_ShowDimmerData

Funktion gibt alle gespeicherten Dimmer-Daten auf dem Display aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_ShowDimmerData ( void )
{
   /* ----- lokale Variablen ----- */
   char temp_string[255];
   char temp_string2[255];
   unsigned char index;


   /* ----- Programmcode ----- */

   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_ShowDimmerData\n" );
   #endif

   lcd_fill (0);
   set_font (BOLDFONT);
   draw_string (0,0,"Dimmer-Daten", LCD_COLOR_B, DRAW_PUT);

   set_font (SMALLFONT);

   for ( index = 0 ; index < 8 ; index++ )
   {
      // Zahl in 2-stelligen Hex-Wert wandeln
      // Dez2Hex( temp_string2, starlightData.dimmer[ index ].dimmwert, 3 );
      FormatedDez( temp_string2, starlightData.dimmer[ index ].dimmwert, 2 );
      sprintf( temp_string, "Dimmer %d: L%x D%s T%d S%x\n", ( index + 1 ) ,
         starlightData.dimmer[ index ].status ,
         temp_string2,
         starlightData.dimmer[ index ].temperatur,
         starlightData.dimmer[ index ].sound );
      draw_string ( 0, 14 + ( index * 8 ), temp_string, LCD_COLOR_B, DRAW_PUT );
   }

   sprintf( temp_string, "Temp: %d" , starlightUebertemperatur );
   draw_string ( 0, 86, temp_string, LCD_COLOR_B, DRAW_PUT );

   switch ( starlightUebertemperatur_flag )
   {
      case false:
         draw_string ( 80,86,"false", LCD_COLOR_B, DRAW_PUT);
         break;
      case true:
         draw_string ( 80,86,"true", LCD_COLOR_B, DRAW_PUT);
         break;
   }

   draw_string (10,150,"www.kleinmaeusiges.de", LCD_COLOR_B, DRAW_PUT);

    waitKeyUpDown ();

}
/* ----- Ende StarLight_ShowDimmerData ------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_SoundSchalten

Funktion steuert die Wiedergabe von akustischen Feedback von Master und Slave.
Die Funktion ist als Toggelfunktion umgesetzt.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_SoundSchalten ( void )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;

   #if STARLIGHT_COTROL_DEBUG
      char trace_string[255];
   #endif


  /* ----- Programmcode ----- */
   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_SoundSchalten\n" );
      sprintf( trace_string, "Sound-Status: %x\n", starlightData.sound_status ); serial_puts( trace_string );
   #endif

   // Sound-Status Toggeln
   switch( starlightData.sound_status )
   {
      case FKT_STATUS_AUS:
         starlightData.sound_status = FKT_STATUS_EIN;
         break;

      case FKT_STATUS_EIN:
         starlightData.sound_status = FKT_STATUS_AUS;
         break;

      default:
         break;
   }

   // Soundstatus an Master bertragen
   result_value = RfStarLightSoundSchalten ( starlightData.sound_status );

   // Prfen ob Funktion korrekt ausgefhrt wurde
   if ( result_value == true )
   {
      // JA, dann kann aus ACK Botschaft noch Temperaturinformation extrahiert werden.
      
      // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
      starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
      starlightUebertemperatur_flag = true;
   }

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

}
/* ----- Ende StarLight_SoundSchalten -------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_AlleLampenSchalten

Funktion steuert alle Lampen auf einmal. Alle Lampen knnen auf einmal EIN bzg.
AUS geschaltet werden

bergabeparameter:      lampen_status = [ FKT_STATUS_AUS / FKT_STATUS_EIN ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_AlleLampenSchalten ( unsigned char lampen_status )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;
  unsigned char index;

   #if STARLIGHT_COTROL_DEBUG
      char trace_string[255];
   #endif


  /* ----- Programmcode ----- */
   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_AlleLampenSchalten\n" );
      sprintf( trace_string, "Lampen-Status: %x\n", lampen_status ); serial_puts( trace_string );
   #endif

   // Steuerkommando an Master bertragen
   result_value = RfStarLightGesamtlicht ( lampen_status );

   // Nun auswerten ob Kommunikation erfolgreich war oder nicht

   if ( result_value == true )
   {
      // JA, Kommunikation erfolgte ohne Fehler

      // Wenn Beleuchtungsanlage AUS war dann kommt Kommando = ACK mit 0x00 zurck
      // Da Kommando Broadcastmessages auslst wird mit Kommando = ACK mit 0x01 positiv quitiert!

      // Prfen ob ACK Botschaft empfangen
      if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
      {
         // JA - Nun noch prfen ob Statusbyte auf 0x00
         if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
         {
            // JA = Anlage AUS

            // Fehlerbehandlung
            msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lichtanlage ist AUS  \n");
            drawMainscreen ( );
            result_value = false;

         } // FKT_STATUS_AUS
         else
         {
            // NEIN = Anlage EIN

            // Alle Daten auch fr interne Datenstruktur setzen
            for ( index = 0 ; index < 8 ; index++ )
            {
               starlightData.dimmer[ index ].status   = lampen_status;
            } // Schleife ber alle Dimmer
         } // FKT_STATUS_EIN

         // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
         starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightUebertemperatur_flag = true;

      } // ACKNOWLEDGE empfangen
   } // Kommunikation erfolgreich
   else
   {
      // NEIN, Kommunikation erfolgte mit Fehler
   } // Kommunikation fehlerhaft

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

   return result_value;
}
/* ----- Ende StarLight_AlleLampenSchalten --------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_LampenDimmen

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen gesteuert
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        dimmwert = [ 1..31 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_LampenDimmen ( unsigned char dimmer, unsigned char dimmwert )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;
  unsigned char index;

   #if STARLIGHT_COTROL_DEBUG
      char trace_string[255];
   #endif


  /* ----- Programmcode ----- */
   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_LampenDimmen\n" );
      sprintf( trace_string, "Dimmer: %x  -  Dimmwert: %x\n", dimmer , dimmwert ); serial_puts( trace_string );
   #endif

   // Steuerkommando an Master bertragen
   // BOOL RfStarLightDimmerSteuern ( unsigned char dimmer , unsigned char dimmwert )
   result_value = RfStarLightDimmerSteuern ( dimmer , dimmwert );

   // Nun auswerten ob Kommunikation erfolgreich war oder nicht

   if ( result_value == true )
   {
      // JA, Kommunikation erfolgte ohne Fehler

      // Wenn Beleuchtungsanlage AUS war dann kommt Kommando = ACK mit 0x00 zurck
      // Da Kommando Broadcastmessages auslsen kann wird entweder Status gesendet oder mit Kommando = ACK mit 0x01 positiv quitiert!

      // Prfen ob ACK Botschaft empfangen
      if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
      {
         // JA - Nun noch prfen ob Statusbyte auf 0x00
         if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
         {
            // JA = Anlage AUS

            // Fehlerbehandlung
            msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lichtanlage ist AUS  \n");
            result_value = false;

         } // FKT_STATUS_AUS
         else
         {
            // NEIN = Anlage EIN

            // Alle Daten auch fr interne Datenstruktur setzen
            for ( index = 0 ; index < 8 ; index++ )
            {
               starlightData.dimmer[ index ].dimmwert = dimmwert;
            } // Schleife ber alle Dimmer
         } // FKT_STATUS_EIN

         // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
         starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightUebertemperatur_flag = true;

      } // ACKNOWLEDGE empfangen
      else
      {
         // NEIN - Es wurde ein einzelner Dimmer angesprochen und als Ergbnise kam Status zurck

         // Datenstrukturen auslesen und Daten in Strukt eintragen zur weiteren Verarbeitung
         starlightData.dimmer[ ( dimmer - 1 ) ].status     = rfStarLightData.RxData.RxData.dimmer;
         starlightData.dimmer[ ( dimmer - 1 ) ].dimmwert   = rfStarLightData.RxData.RxData.dimmwert;
         starlightData.dimmer[ ( dimmer - 1 ) ].temperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightData.dimmer[ ( dimmer - 1 ) ].sound      = rfStarLightData.RxData.RxData.sound;

      } // Einzelner Dimmer angesprochen -> Statusinformation erhalten
   } // Kommunikation erfolgreich
   else
   {
      // NEIN, Kommunikation erfolgte mit Fehler
   } // Kommunikation fehlerhaft

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

   return result_value;
}
/* ----- Ende StarLight_LampenDimmen --------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_LampenSchalten

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen geschaltet
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        dimmwert = [ 0..1 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_LampenSchalten ( unsigned char dimmer, unsigned char status )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;
  unsigned char index;

   #if STARLIGHT_COTROL_DEBUG
      char trace_string[255];
   #endif


  /* ----- Programmcode ----- */
   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_LampenSchalten\n" );
      sprintf( trace_string, "Dimmer: %x  -  Status: %x\n", dimmer , status ); serial_puts( trace_string );
   #endif

   // Steuerkommando an Master bertragen
   result_value = RfStarLightDimmerSchalten ( dimmer , status );

   // Nun auswerten ob Kommunikation erfolgreich war oder nicht

   if ( result_value == true )
   {
      // JA, Kommunikation erfolgte ohne Fehler

      // Wenn Beleuchtungsanlage AUS war dann kommt Kommando = ACK mit 0x00 zurck
      // Da Kommando Broadcastmessages auslsen kann wird entweder Status gesendet oder mit Kommando = ACK mit 0x01 positiv quitiert!

      // Prfen ob ACK Botschaft empfangen
      if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
      {
         // JA - Nun noch prfen ob Statusbyte auf 0x00
         if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
         {
            // JA = Anlage AUS

            // Fehlerbehandlung
            msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lichtanlage ist AUS  \n");
            result_value = false;

         } // FKT_STATUS_AUS
         else
         {
            // NEIN = Anlage EIN

            // Alle Daten auch fr interne Datenstruktur setzen
            for ( index = 0 ; index < 8 ; index++ )
            {
               starlightData.dimmer[ index ].status = status;
            } // Schleife ber alle Dimmer
         } // FKT_STATUS_EIN

         // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
         starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightUebertemperatur_flag = true;

      } // ACKNOWLEDGE empfangen
      else
      {
         // NEIN - Es wurde ein einzelner Dimmer angesprochen und als Ergbnise kam Status zurck

         // Datenstrukturen auslesen und Daten in Strukt eintragen zur weiteren Verarbeitung
         starlightData.dimmer[ ( dimmer - 1 ) ].status     = rfStarLightData.RxData.RxData.dimmer;
         starlightData.dimmer[ ( dimmer - 1 ) ].dimmwert   = rfStarLightData.RxData.RxData.dimmwert;
         starlightData.dimmer[ ( dimmer - 1 ) ].temperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightData.dimmer[ ( dimmer - 1 ) ].sound      = rfStarLightData.RxData.RxData.sound;

      } // Einzelner Dimmer angesprochen -> Statusinformation erhalten
   } // Kommunikation erfolgreich
   else
   {
      // NEIN, Kommunikation erfolgte mit Fehler
   } // Kommunikation fehlerhaft

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

   return result_value;
}
/* ----- Ende StarLight_LampenDimmen --------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_DimmerSchalten

Mittels dieser Funktion knnen die Dimmer einzeln oder zusammen geschaltet
werden.

bergabeparameter:      dimmer   = [ 1..8 / 0xFF ]
                        status   = [ 0..1 ]
                        dimmwert = [ 1.. 31 ]

Rckgabeparameter:      TRUE  = Funktion war erfolgreich
                        FALSE = Es ist ein Fehler aufgetreten bzw. Anlage
                                war abgeschaltet.
---------------------------------------------------------------------------- */
BOOL StarLight_DimmerSchalten ( unsigned char dimmer, unsigned char status, unsigned char dimmwert )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;
  unsigned char index;

   #if STARLIGHT_COTROL_DEBUG
      char trace_string[255];
   #endif


  /* ----- Programmcode ----- */
   #if STARLIGHT_COTROL_DEBUG
      serial_puts( "Enter StarLight_DimmerSchalten\n" );
      sprintf( trace_string, "Dimmer: %x  -  Status: %x  -  Dimmwert: %x\n", dimmer , status, dimmwert ); serial_puts( trace_string );
   #endif

   // Steuerkommando an Master bertragen
   // BOOL RfStarLightDimmerGesamt ( unsigned char dimmer , unsigned char dimmstatus , unsigned char dimmwert )

   result_value = RfStarLightDimmerGesamt ( dimmer , status, dimmwert );

   // Nun auswerten ob Kommunikation erfolgreich war oder nicht

   if ( result_value == true )
   {
      // JA, Kommunikation erfolgte ohne Fehler

      // Wenn Beleuchtungsanlage AUS war dann kommt Kommando = ACK mit 0x00 zurck
      // Da Kommando Broadcastmessages auslsen kann wird entweder Status gesendet oder mit Kommando = ACK mit 0x01 positiv quitiert!

      // Prfen ob ACK Botschaft empfangen
      if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
      {
         // JA - Nun noch prfen ob Statusbyte auf 0x00
         if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
         {
            // JA = Anlage AUS

            // Fehlerbehandlung
            msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lichtanlage ist AUS  \n");
            result_value = false;

         } // FKT_STATUS_AUS
         else
         {
            // NEIN = Anlage EIN

            // Alle Daten auch fr interne Datenstruktur setzen
            for ( index = 0 ; index < 8 ; index++ )
            {
               starlightData.dimmer[ index ].status = status;
               starlightData.dimmer[ index ].dimmwert = dimmwert;
            } // Schleife ber alle Dimmer
         } // FKT_STATUS_EIN

         // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
         starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightUebertemperatur_flag = true;

      } // ACKNOWLEDGE empfangen
      else
      {
         // NEIN - Es wurde ein einzelner Dimmer angesprochen und als Ergbnise kam Status zurck

         // Datenstrukturen auslesen und Daten in Strukt eintragen zur weiteren Verarbeitung
         starlightData.dimmer[ ( dimmer - 1 ) ].status     = rfStarLightData.RxData.RxData.dimmer;
         starlightData.dimmer[ ( dimmer - 1 ) ].dimmwert   = rfStarLightData.RxData.RxData.dimmwert;
         starlightData.dimmer[ ( dimmer - 1 ) ].temperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightData.dimmer[ ( dimmer - 1 ) ].sound      = rfStarLightData.RxData.RxData.sound;

      } // Einzelner Dimmer angesprochen -> Statusinformation erhalten
   } // Kommunikation erfolgreich
   else
   {
      // NEIN, Kommunikation erfolgte mit Fehler
   } // Kommunikation fehlerhaft

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

   return result_value;
}
/* ----- Ende StarLight_DimmerSchalten -------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Demomodus

Funktion steuert Demomodus an

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
BOOL StarLight_Demomodus ( void )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;


  /* ----- Programmcode ----- */

   // Soundstatus an Master bertragen
   result_value = RfStarLightDemomodus(  );

   // Nun auswerten ob Kommunikation erfolgreich war oder nicht

   if ( result_value == true )
   {
      // JA, Kommunikation erfolgte ohne Fehler

      // Wenn Beleuchtungsanlage AUS war dann kommt Kommando = ACK mit 0x00 zurck
      // Da Kommando Broadcastmessages auslst wird mit Kommando = ACK mit 0x01 positiv quitiert!

      // Prfen ob ACK Botschaft empfangen
      if ( rfStarLightData.RxData.RxData.kommando == ACKNOWLEDGE )
      {
         // JA - Nun noch prfen ob Statusbyte auf 0x00
         if ( rfStarLightData.RxData.RxData.dimmer == FKT_STATUS_AUS )
         {
            // JA = Anlage AUS

            // Fehlerbehandlung
            msgbox (45, BC_OKOnly | BC_DefaultButton1, "\n  Lichtanlage ist AUS  \n");
            drawMainscreen ( );

            result_value = false;

         } // FKT_STATUS_AUS

         // In Statusbotschaft enthaltene Temperaturschwelle uebernehmen
         starlightUebertemperatur = rfStarLightData.RxData.RxData.temperatur;
         starlightUebertemperatur_flag = true;

      } // ACKNOWLEDGE empfangen
   } // Kommunikation erfolgreich

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

   return result_value;
}
/* ----- Ende StarLight_Demomodus ------------------------------------------ */


/* ----------------------------------------------------------------------------
Funktion: StarLight_AnlageSchalten

Funktion ist Toggelfunktion und schaltet abhngig vom Zustand System ein und
aus. Beim Wiedereinschalten wird LUM gesetzt.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_AnlageSchalten ( void )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;


  /* ----- Programmcode ----- */

   // Soundstatus an Master bertragen
   result_value = RfStarLightLichtanlageSchalten(  );

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

}
/* ----- Ende StarLight_AnlageSchalten ------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_DrawDimmerDisplay

Funktion zeichnet die Betriebsdaten fr einen Dimmer

bergabeparameter:      dimmerData = aktiver Dimmer

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DrawDimmerDisplay( unsigned int dimmerData )
{
   /* ----- lokale Variablen ----- */
   char           result_string[ 255 ];
   unsigned char  balken_pos_x; // x-Position
   unsigned char  balken_pos_y; // y-Position
   unsigned char  balken_pos_b; // Breite
   unsigned char  balken_pos_h; // Hoehe

   unsigned char  temp_balkenhoehe;
   float          temp_float;



   /* ----- Programmcode ----- */

   // Display lschen
   lcd_fill ( 0 );

   // Dialog aufbauen
   set_font ( SMALLFONT );

   // Header ausgeben
   sprintf( result_string, "Dimmer %d", dimmerData );
   draw_string ( STARLIGHT_DIMMER_POS_X, STARLIGHT_DIMMER_POS_Y, result_string, LCD_COLOR_B, DRAW_PUT);

   // Rahmen fr Balken zeichnen
   draw_rect ( STARLIGHT_BALKEN_POS_X, STARLIGHT_BALKEN_POS_Y - 2,
               STARLIGHT_BALKEN_POS_BREITE, STARLIGHT_BALKEN_POS_HOEHE + 3,
               STARLIGHT_BALKEN_RAHMENBREITE,
               LCD_COLOR_B, DRAW_PUT);

   // Balken zeichnen
   balken_pos_x = STARLIGHT_BALKEN_POS_X + STARLIGHT_BALKEN_ABSTAND;
   balken_pos_b = STARLIGHT_BALKEN_POS_BREITE - ( STARLIGHT_BALKEN_RAHMENBREITE * STARLIGHT_BALKEN_ABSTAND );

   temp_float       = ( STARLIGHT_BALKEN_POS_HOEHE - ( STARLIGHT_BALKEN_RAHMENBREITE * STARLIGHT_BALKEN_ABSTAND)) / 30.0 * starlightData.dimmer[ dimmerData - 1 ].dimmwert;
   temp_balkenhoehe = (unsigned char) temp_float;

   balken_pos_y = ( STARLIGHT_BALKEN_POS_Y + STARLIGHT_BALKEN_ABSTAND ) + 1 +
                  ( STARLIGHT_BALKEN_POS_HOEHE - ( STARLIGHT_BALKEN_RAHMENBREITE * STARLIGHT_BALKEN_ABSTAND ) ) -
                  temp_balkenhoehe;

   balken_pos_h = temp_balkenhoehe;

   draw_block  ( balken_pos_x, balken_pos_y, balken_pos_b, balken_pos_h, LCD_COLOR_DG, DRAW_PUT);

   // Schriftart FETT und Beschriftung des Balkens schreiben
   set_font ( BOLDFONT );
   draw_string ( STARLIGHT_BESCHRIFTUNG_LINKS_POS_X, STARLIGHT_BALKEN_POS_Y,
                 "32", LCD_COLOR_B, DRAW_PUT);
   draw_string ( STARLIGHT_BESCHRIFTUNG_LINKS_POS_X + STARLIGHT_BESCHRIFTUNG_LINKS_DELTA_X, STARLIGHT_BALKEN_POS_Y + STARLIGHT_BALKEN_POS_HOEHE - STARLIGHT_BESCHRIFTUNG_LINKS_DELTA_Y,
                 "0", LCD_COLOR_B, DRAW_PUT);
   draw_string ( STARLIGHT_BESCHRIFTUNG_RECHTS_POS_X, STARLIGHT_BALKEN_POS_Y,
                 "100%", LCD_COLOR_B, DRAW_PUT);
   draw_string ( STARLIGHT_BESCHRIFTUNG_RECHTS_POS_X, STARLIGHT_BALKEN_POS_Y + STARLIGHT_BALKEN_POS_HOEHE - STARLIGHT_BESCHRIFTUNG_LINKS_DELTA_Y,
                 "0%", LCD_COLOR_B, DRAW_PUT);

   // Schriftart GROSS FETT und Dimmerdaten auf Display schreiben
   set_font ( BOLDDOUBLEFONT );

   FormatedDez( result_string, starlightData.dimmer[ dimmerData - 1 ].dimmwert, 2 );
   draw_string ( STARLIGHT_BALKEN_POS_X + STARLIGHT_BALKEN_TEXT_DELTA_X, STARLIGHT_BALKEN_POS_Y - STARLIGHT_BALKEN_TEXT_DELTA_Y, result_string, LCD_COLOR_B, DRAW_PUT);

   draw_numU ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_DIMM_POS_Y, starlightData.dimmer[ dimmerData - 1 ].dimmwert,   0, LCD_COLOR_B, DRAW_PUT );
   switch ( starlightData.dimmer[ dimmerData - 1 ].status )
   {
      case STARLIGHT_FUNKTION_AUS:
         sprintf( result_string, "AUS" );
         break;
      case STARLIGHT_FUNKTION_EIN:
         sprintf( result_string, "EIN" );
         break;
   }
   draw_string ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_STAT_POS_Y, result_string, LCD_COLOR_B, DRAW_PUT);
   draw_numU ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_TEMP_POS_Y, starlightData.dimmer[ dimmerData - 1 ].temperatur, 0, LCD_COLOR_B, DRAW_PUT );

   // Schriftart SMALL und berschriften schreiben
   set_font ( SMALLFONT );
   draw_string ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_DIMM_POS_Y - STARLIGHT_BESCHRIFTUNG_TEXT_DELTA,
                 "Dimmwert:", LCD_COLOR_B, DRAW_PUT);
   draw_string ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_STAT_POS_Y - STARLIGHT_BESCHRIFTUNG_TEXT_DELTA,
                 "Schalter:", LCD_COLOR_B, DRAW_PUT);
   draw_string ( STARLIGHT_BESCHRIFTUNG_TEXT_POS_X, STARLIGHT_BESCHRIFTUNG_TEXT_TEMP_POS_Y - STARLIGHT_BESCHRIFTUNG_TEXT_DELTA,
                 "Temperatur:", LCD_COLOR_B, DRAW_PUT);
}
/* ----- Ende StarLight_DrawDimmerDisplay ---------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_DialogDimmer

Funktion steuert nach Aufruf der Zifferntasten 1-8 die Dialogfhrung mit
grafischer Benutzerfhrung des Dimmers.

Untersttzte Tasten sind:
   Exit    = Verlassen / Ende
   1-8     = Auswahl eines neuen Dimmers
   VOL+/-  = Dimmstufe verndern
   PROG+/- = anderen Dimmer auswhlen
   ROT     = ausgewhlter Dimmer aus
   GRN    = ausgewhlter Dimmer ein
   MEN    = aufruf des Dimmer-Einstellmens
   A-D     = ShortCuts je nach Programmierung
   OK      = Daten werden zum Dimmer gesendet

bergabeparameter:      dimmer = Dimmer der gesteuert werden soll

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DialogDimmer ( unsigned int dimmerData )
{

   /* ----- lokale Variablen ----- */
   unsigned char  dimmer_index;
   unsigned char  result_value;
   unsigned char  changed_flag_dimmer;
   unsigned char  changed_flag_status;


   /* ----- Programmcode ----- */

   // Dimmer in Arbeitsvariable laden
   dimmer_index = dimmerData;
   changed_flag_dimmer = false;
   changed_flag_status = false;


   // Timer fr Men aufziehen
   addTimeout ( &menu_timer, MENU_SEQUENCE_TIMEOUT );


   // Grafik aufbauen
   StarLight_DrawDimmerDisplay( dimmer_index );

   while (KEY_Exit) {};
   waitKeyUp ();
   while (1)
   {

      update_active_ind ();

      //waitKeyUpDown ();
      // Wait until key pressed instead.
      cpu_idle ();
      if (isKeyPressed ())
      {
         setBacklight (BL_AUTO);    // pwm value

         addTimeout ( &menu_timer, MENU_SEQUENCE_TIMEOUT );

         // Credits ausgeben
         if (KEY_Info)
         {
            // Grafik erneut aufbauen
            Starlight_Credits (  );
         }

         // Dialog verlassen
         else if ( KEY_Exit )
         {

            // Timeout aus Scheduler-Queue lschen
            removeTimeout( &menu_timer );
            menu_timer = 0;

            // Men verlassen
            return;
         }

         // Daten zum Dimmer senden (nderung kann nur Dimmstufe betreffen)
         else if (KEY_OK)
         {
            // ggf. Dimmwerte setzen (an Master bertragen)
            if ( changed_flag_dimmer == true )
            {
               result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
               changed_flag_dimmer = false;
            }

            // ggf. Status setzen (an Master bertragen)
            if ( changed_flag_status == true )
            {
               result_value = StarLight_LampenSchalten ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].status );
               changed_flag_status = false;
            }
         }

         // Dimmwert erhhen aber noch nicht an Dimmer senden, erst mit OK
         else if ( KEY_Up )
         {
            starlightData.dimmer[ dimmer_index - 1 ].dimmwert++;
            if ( starlightData.dimmer[ dimmer_index - 1 ].dimmwert > STARLIGHT_DIMMER_DATA_MAX )
            {
               starlightData.dimmer[ dimmer_index - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MAX;
            }
            changed_flag_dimmer = true;
         }

         // Dimmwert verringern aber noch nicht an Dimmer senden, erst mit OK
         else if ( KEY_Down )
         {
            starlightData.dimmer[ dimmer_index - 1 ].dimmwert--;
            if ( starlightData.dimmer[ dimmer_index - 1 ].dimmwert < STARLIGHT_DIMMER_DATA_MIN )
            {
               starlightData.dimmer[ dimmer_index - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MIN;
            }
            changed_flag_dimmer = true;
         }

         // Dimmwert erhhen und Update gleich an Dimmer senden
         else if ( KEY_Vplus )
         {
            starlightData.dimmer[ dimmer_index - 1 ].dimmwert++;
            if ( starlightData.dimmer[ dimmer_index - 1 ].dimmwert > STARLIGHT_DIMMER_DATA_MAX )
            {
               starlightData.dimmer[ dimmer_index - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MAX;
            }
            result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
         }

         // Dimmwert verringern und Update gleich an Dimmer senden
         else if ( KEY_Vminus )
         {
            starlightData.dimmer[ dimmer_index - 1 ].dimmwert--;
            if ( starlightData.dimmer[ dimmer_index - 1 ].dimmwert < STARLIGHT_DIMMER_DATA_MIN )
            {
               starlightData.dimmer[ dimmer_index - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MIN;
            }
            result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
         }

         // Dimmerstatus mit links/rechts schalten
         else if ( KEY_Left || KEY_Right )
         {
            switch ( starlightData.dimmer[ dimmer_index - 1 ].status )
            {
               case STARLIGHT_FUNKTION_AUS:
                  starlightData.dimmer[ dimmer_index - 1 ].status = STARLIGHT_FUNKTION_EIN;
                  break;
               case STARLIGHT_FUNKTION_EIN:
                  starlightData.dimmer[ dimmer_index - 1 ].status = STARLIGHT_FUNKTION_AUS;
                  break;
            }
            changed_flag_status = true;
         }

         // Dimmer runter schalten
         else if ( KEY_Pminus )
         {

            // ggf. Dimmwerte setzen (an Master bertragen)
            if ( changed_flag_dimmer == true )
            {
               result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
               changed_flag_dimmer = false;
            }

            // ggf. Status setzen (an Master bertragen)
            if ( changed_flag_status == true )
            {
               result_value = StarLight_LampenSchalten ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].status );
               changed_flag_status = false;
            }

            dimmer_index--;
            if ( dimmer_index < 1 )
            {
               dimmer_index = 1;
            }
         }

         // Dimmer hochschalten
         else if ( KEY_Pplus )
         {

            // ggf. Dimmwerte setzen (an Master bertragen)
            if ( changed_flag_dimmer == true )
            {
               result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
               changed_flag_dimmer = false;
            }

            // ggf. Status setzen (an Master bertragen)
            if ( changed_flag_status == true )
            {
               result_value = StarLight_LampenSchalten ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].status );
               changed_flag_status = false;
            }

            dimmer_index++;
            if ( dimmer_index > 8 )
            {
               dimmer_index = 8;
            }
         }

         // Dimmer EIN und 33%
         else if (KEY_VTX1)
         {
            StarLight_DimmerSchalten( dimmer_index, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_33);
//            if ( StarLight_LampenDimmen ( dimmer_index, STARLIGHT_DIMMER_33 ) == true ) // Dimmer einstellen
//            {
//               // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//               // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//
//               // Prfen ob Dimmer noch aus
//               if ( starlightData.dimmer[ dimmer_index - 1 ].status == STARLIGHT_FUNKTION_AUS )
//               {
//                  // JA, dann auf jeden Fall Dimmer einschalten
//                  result_value = StarLight_LampenSchalten ( dimmer_index, STARLIGHT_FUNKTION_EIN ); // Auf jeden Fall Lampen einschalten
//               }
//            }
         }

         // Dimmer EIN und 66%
         else if (KEY_VTX2)
         {
            StarLight_DimmerSchalten( dimmer_index, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_66);
//            if ( StarLight_LampenDimmen ( dimmer_index, STARLIGHT_DIMMER_66 ) == true ) // Dimmer einstellen
//            {
//               // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//               // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//
//               // Prfen ob Dimmer noch aus
//               if ( starlightData.dimmer[ dimmer_index - 1 ].status == STARLIGHT_FUNKTION_AUS )
//               {
//                  // JA, dann auf jeden Fall Dimmer einschalten
//                  result_value = StarLight_LampenSchalten ( dimmer_index, STARLIGHT_FUNKTION_EIN ); // Auf jeden Fall Lampen einschalten
//               }
//            }
         }

         // Dimmer EIN und 100%
         else if (KEY_VTX3)
         {
            StarLight_DimmerSchalten( dimmer_index, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_100);
//            if ( StarLight_LampenDimmen ( dimmer_index, STARLIGHT_DIMMER_100 ) == true ) // Dimmer einstellen
//            {
//               // Webb LampenDimmen mit Fehler antwortet dann mssen Lamen auch nicht geschaltet werden
//               // Wichtig um z.B. Anlage AUS abzufangen und zustzliches PopUp zu vermeiden
//
//               // Prfen ob Dimmer noch aus
//               if ( starlightData.dimmer[ dimmer_index - 1 ].status == STARLIGHT_FUNKTION_AUS )
//               {
//                  // JA, dann auf jeden Fall Dimmer einschalten
//                  result_value = StarLight_LampenSchalten ( dimmer_index, STARLIGHT_FUNKTION_EIN ); // Auf jeden Fall Lampen einschalten
//               }
//            }
         }

         // Programm A abrufen
         else if (KEY_A)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_A );
         }

         // Programm B abrufen
         else if (KEY_B)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_B );
         }

         // Programm C abrufen
         else if (KEY_C)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_C );
         }

         // Programm D abrufen
         else if (KEY_D)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_D );
         }

         // Dimmer umschalten
         else if ( KEY_1 || KEY_2 || KEY_3 || KEY_4 || KEY_5 || KEY_6 || KEY_7 || KEY_8 )
         {

            // ggf. Dimmwerte setzen (an Master bertragen)
            if ( changed_flag_dimmer == true )
            {
               result_value = StarLight_LampenDimmen ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].dimmwert );
               changed_flag_dimmer = false;
            }

            // ggf. Status setzen (an Master bertragen)
            if ( changed_flag_status == true )
            {
               result_value = StarLight_LampenSchalten ( dimmer_index, starlightData.dimmer[ dimmer_index - 1 ].status );
               changed_flag_status = false;
            }

            // KEY's 1-8 auswerten
            if ( KEY_1 )      { dimmer_index = STARLIGHT_DIMMER_1; }
            else if ( KEY_2 ) { dimmer_index = STARLIGHT_DIMMER_2; }
            else if ( KEY_3 ) { dimmer_index = STARLIGHT_DIMMER_3; }
            else if ( KEY_4 ) { dimmer_index = STARLIGHT_DIMMER_4; }
            else if ( KEY_5 ) { dimmer_index = STARLIGHT_DIMMER_5; }
            else if ( KEY_6 ) { dimmer_index = STARLIGHT_DIMMER_6; }
            else if ( KEY_7 ) { dimmer_index = STARLIGHT_DIMMER_7; }
            else if ( KEY_8 ) { dimmer_index = STARLIGHT_DIMMER_8; }
         }

         // Lampe EIN des aktuellen Dimmers
         else if (KEY_Green )
         {
            // Prfen ob Dimmer noch aus (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index - 1 ].status == STARLIGHT_FUNKTION_AUS )
            {
               // JA, dann auf jeden Fall Dimmer einschalten
               result_value = StarLight_LampenSchalten ( dimmer_index, STARLIGHT_FUNKTION_EIN );
            }
         }

         // Lampe AUS des aktuellen Dimmers
         else if (KEY_Red)
         {
            // Prfen ob Dimmer noch ein (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index - 1 ].status == STARLIGHT_FUNKTION_EIN )
            {
               // JA, dann auf jeden Fall Dimmer ausschalten
               result_value = StarLight_LampenSchalten ( dimmer_index, STARLIGHT_FUNKTION_AUS );
            }
         }

         // Anzeige aller Dimmerdaten
         else if (KEY_TV)
         {
            StarLight_ShowDimmerData( );
         }

         // Anlage AUS/AN schalten
         else if (KEY_Power)
         {
            StarLight_AnlageSchalten(  );
         }

         // Tastenpaar fr ALLE Lampen EIN/AUS
         else if ( KEY_Yellow || KEY_Blue )
         {
            // Steuerung fr alle Lampen EIN/AUS
            if ( KEY_Yellow )
            {
               StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );
            }
            else if ( KEY_Blue )
            {
               StarLight_AlleLampenSchalten ( FKT_STATUS_AUS );
            }
         }

         // Dimmer Einstellmen aufrufen
         else if (KEY_Menu)
         {
            menu_exec (&starlightSettingsMenu);
         }

         // Sound schalten
         else if (KEY_Mute)
         {
            StarLight_SoundSchalten (  );
         }

         // Betty Men anzeigen
         else if (KEY_Betty)
         {
            menu_exec (&mainMenu);
         }


         // Warten bis Taste losgelassen
         waitKeyUp ();

         // Display refresh
         StarLight_DrawDimmerDisplay( dimmer_index );

      }   // if (isKeyPressed ())

      if ( !menu_timer )
      {
         // Timeout aus Scheduler-Queue lschen
         removeTimeout( &menu_timer );
         menu_timer = 0;

         // Men verlassen
         return;
      }
   }   // while (1)


}
/* ----- Ende StarLight_DialogDimmer --------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_A

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_A (void* pData)
{

   /* ----- lokale Variablen ----- */

   const LABEL label1  = { BC_Labelnotab,   0,  15,  15, 10, BOLDFONT,  0, "D1:", 0 };
   const LABEL label2  = { BC_Labelnotab,  20,  15,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox1      = { BC_Numbox,      55,  15,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 0 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label3  = { BC_Labelnotab,  70,  15,  40, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox2      = { BC_Numbox,     110,  15,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 0 ].dimmwert,     0, 0, 1, 31};

   const LABEL label4  = { BC_Labelnotab,   0,  25,  15, 10, BOLDFONT,  0, "D2:", 0 };
   const LABEL label5  = { BC_Labelnotab,  20,  25,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox3      = { BC_Numbox,      55,  25,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 1 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label6  = { BC_Labelnotab,  70,  25,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox4      = { BC_Numbox,     110,  25,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 1 ].dimmwert,     0, 0, 1, 31};

   const LABEL label7  = { BC_Labelnotab,   0,  35,  15, 10, BOLDFONT,  0, "D3:", 0 };
   const LABEL label8  = { BC_Labelnotab,  20,  35,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox5      = { BC_Numbox,      55,  35,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 2 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label9  = { BC_Labelnotab,  70,  35,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox6      = { BC_Numbox,     110,  35,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 2 ].dimmwert,     0, 0, 1, 31};

   const LABEL label10 = { BC_Labelnotab,   0,  45,  15, 10, BOLDFONT,  0, "D4:", 0 };
   const LABEL label11 = { BC_Labelnotab,  20,  45,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox7      = { BC_Numbox,      55,  45,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 3 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label12 = { BC_Labelnotab,  70,  45,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox8      = { BC_Numbox,     110,  45,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 3 ].dimmwert,     0, 0, 1, 31};

   const LABEL label13 = { BC_Labelnotab,   0,  55,  15, 10, BOLDFONT,  0, "D5:", 0 };
   const LABEL label14 = { BC_Labelnotab,  20,  55,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox9      = { BC_Numbox,      55,  55,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 4 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label15 = { BC_Labelnotab,  70,  55,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox10     = { BC_Numbox,     110,  55,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 4 ].dimmwert,     0, 0, 1, 31};

   const LABEL label16 = { BC_Labelnotab,   0,  65,  15, 10, BOLDFONT,  0, "D6:", 0 };
   const LABEL label17 = { BC_Labelnotab,  20,  65,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox11     = { BC_Numbox,      55,  65,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 5 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label18 = { BC_Labelnotab,  70,  65,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox12     = { BC_Numbox,     110,  65,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 5 ].dimmwert,     0, 0, 1, 31};

   const LABEL label19 = { BC_Labelnotab,   0,  75,  15, 10, BOLDFONT,  0, "D7:", 0 };
   const LABEL label20 = { BC_Labelnotab,  20,  75,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox13     = { BC_Numbox,      55,  75,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 6 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label21 = { BC_Labelnotab,  70,  75,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox14     = { BC_Numbox,     110,  75,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 6 ].dimmwert,     0, 0, 1, 31};

   const LABEL label22 = { BC_Labelnotab,   0,  85,  15, 10, BOLDFONT,  0, "D8:", 0 };
   const LABEL label23 = { BC_Labelnotab,  20,  85,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox15     = { BC_Numbox,      55,  85,   8,  9, SMALLFONT, starlightSetting.data[ 0 ][ 7 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label24 = { BC_Labelnotab,  70,  85,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox16     = { BC_Numbox,     110,  85,  16,  9, SMALLFONT, starlightSetting.data[ 0 ][ 7 ].dimmwert,     0, 0, 1, 31};

   const BUTTON button1 = { BC_Button,     39, 110,  50, 15, SMALLFONT, 0, "Speichern", StarLight_Save_Settings_A };

   CONTROL* controls[ 41 ] = {
                                 (CONTROL*)&label1,
                                 (CONTROL*)&label2,
                                 (CONTROL*)&numbox1,
                                 (CONTROL*)&label3,
                                 (CONTROL*)&numbox2,
                                 (CONTROL*)&label4,
                                 (CONTROL*)&label5,
                                 (CONTROL*)&numbox3,
                                 (CONTROL*)&label6,
                                 (CONTROL*)&numbox4,
                                 (CONTROL*)&label7,
                                 (CONTROL*)&label8,
                                 (CONTROL*)&numbox5,
                                 (CONTROL*)&label9,
                                 (CONTROL*)&numbox6,
                                 (CONTROL*)&label10,
                                 (CONTROL*)&label11,
                                 (CONTROL*)&numbox7,
                                 (CONTROL*)&label12,
                                 (CONTROL*)&numbox8,
                                 (CONTROL*)&label13,
                                 (CONTROL*)&label14,
                                 (CONTROL*)&numbox9,
                                 (CONTROL*)&label15,
                                 (CONTROL*)&numbox10,
                                 (CONTROL*)&label16,
                                 (CONTROL*)&label17,
                                 (CONTROL*)&numbox11,
                                 (CONTROL*)&label18,
                                 (CONTROL*)&numbox12,
                                 (CONTROL*)&label19,
                                 (CONTROL*)&label20,
                                 (CONTROL*)&numbox13,
                                 (CONTROL*)&label21,
                                 (CONTROL*)&numbox14,
                                 (CONTROL*)&label22,
                                 (CONTROL*)&label23,
                                 (CONTROL*)&numbox15,
                                 (CONTROL*)&label24,
                                 (CONTROL*)&numbox16,
                                 (CONTROL*)&button1
                            } ;

    FORM form  = {"ShortCut A"," ",0,0,0,0,controls,2,41,0,0};


   /* ----- Programmcode ----- */
    form_exec (&form);

}
/* ----- Ende Starlight_ShortCut_A ----------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_B

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_B (void* pData)
{

   /* ----- lokale Variablen ----- */

   const LABEL label1  = { BC_Labelnotab,   0,  15,  15, 10, BOLDFONT,  0, "D1:", 0 };
   const LABEL label2  = { BC_Labelnotab,  20,  15,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox1      = { BC_Numbox,      55,  15,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 0 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label3  = { BC_Labelnotab,  70,  15,  40, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox2      = { BC_Numbox,     110,  15,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 0 ].dimmwert,     0, 0, 1, 31};

   const LABEL label4  = { BC_Labelnotab,   0,  25,  15, 10, BOLDFONT,  0, "D2:", 0 };
   const LABEL label5  = { BC_Labelnotab,  20,  25,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox3      = { BC_Numbox,      55,  25,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 1 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label6  = { BC_Labelnotab,  70,  25,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox4      = { BC_Numbox,     110,  25,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 1 ].dimmwert,     0, 0, 1, 31};

   const LABEL label7  = { BC_Labelnotab,   0,  35,  15, 10, BOLDFONT,  0, "D3:", 0 };
   const LABEL label8  = { BC_Labelnotab,  20,  35,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox5      = { BC_Numbox,      55,  35,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 2 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label9  = { BC_Labelnotab,  70,  35,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox6      = { BC_Numbox,     110,  35,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 2 ].dimmwert,     0, 0, 1, 31};

   const LABEL label10 = { BC_Labelnotab,   0,  45,  15, 10, BOLDFONT,  0, "D4:", 0 };
   const LABEL label11 = { BC_Labelnotab,  20,  45,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox7      = { BC_Numbox,      55,  45,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 3 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label12 = { BC_Labelnotab,  70,  45,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox8      = { BC_Numbox,     110,  45,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 3 ].dimmwert,     0, 0, 1, 31};

   const LABEL label13 = { BC_Labelnotab,   0,  55,  15, 10, BOLDFONT,  0, "D5:", 0 };
   const LABEL label14 = { BC_Labelnotab,  20,  55,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox9      = { BC_Numbox,      55,  55,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 4 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label15 = { BC_Labelnotab,  70,  55,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox10     = { BC_Numbox,     110,  55,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 4 ].dimmwert,     0, 0, 1, 31};

   const LABEL label16 = { BC_Labelnotab,   0,  65,  15, 10, BOLDFONT,  0, "D6:", 0 };
   const LABEL label17 = { BC_Labelnotab,  20,  65,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox11     = { BC_Numbox,      55,  65,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 5 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label18 = { BC_Labelnotab,  70,  65,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox12     = { BC_Numbox,     110,  65,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 5 ].dimmwert,     0, 0, 1, 31};

   const LABEL label19 = { BC_Labelnotab,   0,  75,  15, 10, BOLDFONT,  0, "D7:", 0 };
   const LABEL label20 = { BC_Labelnotab,  20,  75,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox13     = { BC_Numbox,      55,  75,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 6 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label21 = { BC_Labelnotab,  70,  75,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox14     = { BC_Numbox,     110,  75,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 6 ].dimmwert,     0, 0, 1, 31};

   const LABEL label22 = { BC_Labelnotab,   0,  85,  15, 10, BOLDFONT,  0, "D8:", 0 };
   const LABEL label23 = { BC_Labelnotab,  20,  85,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox15     = { BC_Numbox,      55,  85,   8,  9, SMALLFONT, starlightSetting.data[ 1 ][ 7 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label24 = { BC_Labelnotab,  70,  85,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox16     = { BC_Numbox,     110,  85,  16,  9, SMALLFONT, starlightSetting.data[ 1 ][ 7 ].dimmwert,     0, 0, 1, 31};

   const BUTTON button1 = { BC_Button,     39, 110,  50, 15, SMALLFONT, 0, "Speichern", StarLight_Save_Settings_B };

   CONTROL* controls[ 41 ] = {
                                 (CONTROL*)&label1,
                                 (CONTROL*)&label2,
                                 (CONTROL*)&numbox1,
                                 (CONTROL*)&label3,
                                 (CONTROL*)&numbox2,
                                 (CONTROL*)&label4,
                                 (CONTROL*)&label5,
                                 (CONTROL*)&numbox3,
                                 (CONTROL*)&label6,
                                 (CONTROL*)&numbox4,
                                 (CONTROL*)&label7,
                                 (CONTROL*)&label8,
                                 (CONTROL*)&numbox5,
                                 (CONTROL*)&label9,
                                 (CONTROL*)&numbox6,
                                 (CONTROL*)&label10,
                                 (CONTROL*)&label11,
                                 (CONTROL*)&numbox7,
                                 (CONTROL*)&label12,
                                 (CONTROL*)&numbox8,
                                 (CONTROL*)&label13,
                                 (CONTROL*)&label14,
                                 (CONTROL*)&numbox9,
                                 (CONTROL*)&label15,
                                 (CONTROL*)&numbox10,
                                 (CONTROL*)&label16,
                                 (CONTROL*)&label17,
                                 (CONTROL*)&numbox11,
                                 (CONTROL*)&label18,
                                 (CONTROL*)&numbox12,
                                 (CONTROL*)&label19,
                                 (CONTROL*)&label20,
                                 (CONTROL*)&numbox13,
                                 (CONTROL*)&label21,
                                 (CONTROL*)&numbox14,
                                 (CONTROL*)&label22,
                                 (CONTROL*)&label23,
                                 (CONTROL*)&numbox15,
                                 (CONTROL*)&label24,
                                 (CONTROL*)&numbox16,
                                 (CONTROL*)&button1
                            } ;

    FORM form  = {"ShortCut B"," ",0,0,0,0,controls,2,41,0,0};


   /* ----- Programmcode ----- */
    form_exec (&form);

}
/* ----- Ende Starlight_ShortCut_B ----------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_C

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_C (void* pData)
{

   /* ----- lokale Variablen ----- */

   const LABEL label1  = { BC_Labelnotab,   0,  15,  15, 10, BOLDFONT,  0, "D1:", 0 };
   const LABEL label2  = { BC_Labelnotab,  20,  15,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox1      = { BC_Numbox,      55,  15,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 0 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label3  = { BC_Labelnotab,  70,  15,  40, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox2      = { BC_Numbox,     110,  15,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 0 ].dimmwert,     0, 0, 1, 31};

   const LABEL label4  = { BC_Labelnotab,   0,  25,  15, 10, BOLDFONT,  0, "D2:", 0 };
   const LABEL label5  = { BC_Labelnotab,  20,  25,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox3      = { BC_Numbox,      55,  25,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 1 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label6  = { BC_Labelnotab,  70,  25,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox4      = { BC_Numbox,     110,  25,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 1 ].dimmwert,     0, 0, 1, 31};

   const LABEL label7  = { BC_Labelnotab,   0,  35,  15, 10, BOLDFONT,  0, "D3:", 0 };
   const LABEL label8  = { BC_Labelnotab,  20,  35,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox5      = { BC_Numbox,      55,  35,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 2 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label9  = { BC_Labelnotab,  70,  35,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox6      = { BC_Numbox,     110,  35,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 2 ].dimmwert,     0, 0, 1, 31};

   const LABEL label10 = { BC_Labelnotab,   0,  45,  15, 10, BOLDFONT,  0, "D4:", 0 };
   const LABEL label11 = { BC_Labelnotab,  20,  45,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox7      = { BC_Numbox,      55,  45,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 3 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label12 = { BC_Labelnotab,  70,  45,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox8      = { BC_Numbox,     110,  45,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 3 ].dimmwert,     0, 0, 1, 31};

   const LABEL label13 = { BC_Labelnotab,   0,  55,  15, 10, BOLDFONT,  0, "D5:", 0 };
   const LABEL label14 = { BC_Labelnotab,  20,  55,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox9      = { BC_Numbox,      55,  55,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 4 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label15 = { BC_Labelnotab,  70,  55,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox10     = { BC_Numbox,     110,  55,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 4 ].dimmwert,     0, 0, 1, 31};

   const LABEL label16 = { BC_Labelnotab,   0,  65,  15, 10, BOLDFONT,  0, "D6:", 0 };
   const LABEL label17 = { BC_Labelnotab,  20,  65,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox11     = { BC_Numbox,      55,  65,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 5 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label18 = { BC_Labelnotab,  70,  65,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox12     = { BC_Numbox,     110,  65,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 5 ].dimmwert,     0, 0, 1, 31};

   const LABEL label19 = { BC_Labelnotab,   0,  75,  15, 10, BOLDFONT,  0, "D7:", 0 };
   const LABEL label20 = { BC_Labelnotab,  20,  75,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox13     = { BC_Numbox,      55,  75,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 6 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label21 = { BC_Labelnotab,  70,  75,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox14     = { BC_Numbox,     110,  75,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 6 ].dimmwert,     0, 0, 1, 31};

   const LABEL label22 = { BC_Labelnotab,   0,  85,  15, 10, BOLDFONT,  0, "D8:", 0 };
   const LABEL label23 = { BC_Labelnotab,  20,  85,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox15     = { BC_Numbox,      55,  85,   8,  9, SMALLFONT, starlightSetting.data[ 2 ][ 7 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label24 = { BC_Labelnotab,  70,  85,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox16     = { BC_Numbox,     110,  85,  16,  9, SMALLFONT, starlightSetting.data[ 2 ][ 7 ].dimmwert,     0, 0, 1, 31};

   const BUTTON button1 = { BC_Button,     39, 110,  50, 15, SMALLFONT, 0, "Speichern", StarLight_Save_Settings_C };

   CONTROL* controls[ 41 ] = {
                                 (CONTROL*)&label1,
                                 (CONTROL*)&label2,
                                 (CONTROL*)&numbox1,
                                 (CONTROL*)&label3,
                                 (CONTROL*)&numbox2,
                                 (CONTROL*)&label4,
                                 (CONTROL*)&label5,
                                 (CONTROL*)&numbox3,
                                 (CONTROL*)&label6,
                                 (CONTROL*)&numbox4,
                                 (CONTROL*)&label7,
                                 (CONTROL*)&label8,
                                 (CONTROL*)&numbox5,
                                 (CONTROL*)&label9,
                                 (CONTROL*)&numbox6,
                                 (CONTROL*)&label10,
                                 (CONTROL*)&label11,
                                 (CONTROL*)&numbox7,
                                 (CONTROL*)&label12,
                                 (CONTROL*)&numbox8,
                                 (CONTROL*)&label13,
                                 (CONTROL*)&label14,
                                 (CONTROL*)&numbox9,
                                 (CONTROL*)&label15,
                                 (CONTROL*)&numbox10,
                                 (CONTROL*)&label16,
                                 (CONTROL*)&label17,
                                 (CONTROL*)&numbox11,
                                 (CONTROL*)&label18,
                                 (CONTROL*)&numbox12,
                                 (CONTROL*)&label19,
                                 (CONTROL*)&label20,
                                 (CONTROL*)&numbox13,
                                 (CONTROL*)&label21,
                                 (CONTROL*)&numbox14,
                                 (CONTROL*)&label22,
                                 (CONTROL*)&label23,
                                 (CONTROL*)&numbox15,
                                 (CONTROL*)&label24,
                                 (CONTROL*)&numbox16,
                                 (CONTROL*)&button1
                            } ;

    FORM form  = {"ShortCut C"," ",0,0,0,0,controls,2,41,0,0};


   /* ----- Programmcode ----- */
    form_exec (&form);

}
/* ----- Ende Starlight_ShortCut_C ----------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: Starlight_ShortCut_D

Menfunktion fr StarLight Einstellung der Taste A

bergabeparameter:      pData = Universalpointer ( nicht verwendet )

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void Starlight_ShortCut_D (void* pData)
{

   /* ----- lokale Variablen ----- */

   const LABEL label1  = { BC_Labelnotab,   0,  15,  15, 10, BOLDFONT,  0, "D1:", 0 };
   const LABEL label2  = { BC_Labelnotab,  20,  15,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox1      = { BC_Numbox,      55,  15,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 0 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label3  = { BC_Labelnotab,  70,  15,  40, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox2      = { BC_Numbox,     110,  15,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 0 ].dimmwert,     0, 0, 1, 31};

   const LABEL label4  = { BC_Labelnotab,   0,  25,  15, 10, BOLDFONT,  0, "D2:", 0 };
   const LABEL label5  = { BC_Labelnotab,  20,  25,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox3      = { BC_Numbox,      55,  25,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 1 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label6  = { BC_Labelnotab,  70,  25,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox4      = { BC_Numbox,     110,  25,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 1 ].dimmwert,     0, 0, 1, 31};

   const LABEL label7  = { BC_Labelnotab,   0,  35,  15, 10, BOLDFONT,  0, "D3:", 0 };
   const LABEL label8  = { BC_Labelnotab,  20,  35,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox5      = { BC_Numbox,      55,  35,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 2 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label9  = { BC_Labelnotab,  70,  35,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox6      = { BC_Numbox,     110,  35,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 2 ].dimmwert,     0, 0, 1, 31};

   const LABEL label10 = { BC_Labelnotab,   0,  45,  15, 10, BOLDFONT,  0, "D4:", 0 };
   const LABEL label11 = { BC_Labelnotab,  20,  45,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox7      = { BC_Numbox,      55,  45,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 3 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label12 = { BC_Labelnotab,  70,  45,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox8      = { BC_Numbox,     110,  45,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 3 ].dimmwert,     0, 0, 1, 31};

   const LABEL label13 = { BC_Labelnotab,   0,  55,  15, 10, BOLDFONT,  0, "D5:", 0 };
   const LABEL label14 = { BC_Labelnotab,  20,  55,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox9      = { BC_Numbox,      55,  55,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 4 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label15 = { BC_Labelnotab,  70,  55,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox10     = { BC_Numbox,     110,  55,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 4 ].dimmwert,     0, 0, 1, 31};

   const LABEL label16 = { BC_Labelnotab,   0,  65,  15, 10, BOLDFONT,  0, "D6:", 0 };
   const LABEL label17 = { BC_Labelnotab,  20,  65,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox11     = { BC_Numbox,      55,  65,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 5 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label18 = { BC_Labelnotab,  70,  65,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox12     = { BC_Numbox,     110,  65,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 5 ].dimmwert,     0, 0, 1, 31};

   const LABEL label19 = { BC_Labelnotab,   0,  75,  15, 10, BOLDFONT,  0, "D7:", 0 };
   const LABEL label20 = { BC_Labelnotab,  20,  75,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox13     = { BC_Numbox,      55,  75,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 6 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label21 = { BC_Labelnotab,  70,  75,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox14     = { BC_Numbox,     110,  75,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 6 ].dimmwert,     0, 0, 1, 31};

   const LABEL label22 = { BC_Labelnotab,   0,  85,  15, 10, BOLDFONT,  0, "D8:", 0 };
   const LABEL label23 = { BC_Labelnotab,  20,  85,  30, 10, SMALLFONT, 0, "Status",   0 };
   NUMBOX numbox15     = { BC_Numbox,      55,  85,   8,  9, SMALLFONT, starlightSetting.data[ 3 ][ 7 ].dimmerstatus, 0, 0, 0,  1};
   const LABEL label24 = { BC_Labelnotab,  70,  85,  30, 10, SMALLFONT, 0, "Dimmer", 0 };
   NUMBOX numbox16     = { BC_Numbox,     110,  85,  16,  9, SMALLFONT, starlightSetting.data[ 3 ][ 7 ].dimmwert,     0, 0, 1, 31};

   const BUTTON button1 = { BC_Button,     39, 110,  50, 15, SMALLFONT, 0, "Speichern", StarLight_Save_Settings_D };

   CONTROL* controls[ 41 ] = {
                                 (CONTROL*)&label1,
                                 (CONTROL*)&label2,
                                 (CONTROL*)&numbox1,
                                 (CONTROL*)&label3,
                                 (CONTROL*)&numbox2,
                                 (CONTROL*)&label4,
                                 (CONTROL*)&label5,
                                 (CONTROL*)&numbox3,
                                 (CONTROL*)&label6,
                                 (CONTROL*)&numbox4,
                                 (CONTROL*)&label7,
                                 (CONTROL*)&label8,
                                 (CONTROL*)&numbox5,
                                 (CONTROL*)&label9,
                                 (CONTROL*)&numbox6,
                                 (CONTROL*)&label10,
                                 (CONTROL*)&label11,
                                 (CONTROL*)&numbox7,
                                 (CONTROL*)&label12,
                                 (CONTROL*)&numbox8,
                                 (CONTROL*)&label13,
                                 (CONTROL*)&label14,
                                 (CONTROL*)&numbox9,
                                 (CONTROL*)&label15,
                                 (CONTROL*)&numbox10,
                                 (CONTROL*)&label16,
                                 (CONTROL*)&label17,
                                 (CONTROL*)&numbox11,
                                 (CONTROL*)&label18,
                                 (CONTROL*)&numbox12,
                                 (CONTROL*)&label19,
                                 (CONTROL*)&label20,
                                 (CONTROL*)&numbox13,
                                 (CONTROL*)&label21,
                                 (CONTROL*)&numbox14,
                                 (CONTROL*)&label22,
                                 (CONTROL*)&label23,
                                 (CONTROL*)&numbox15,
                                 (CONTROL*)&label24,
                                 (CONTROL*)&numbox16,
                                 (CONTROL*)&button1
                            } ;

    FORM form  = {"ShortCut D"," ",0,0,0,0,controls,2,41,0,0};


   /* ----- Programmcode ----- */
    form_exec (&form);
}
/* ----- Ende Starlight_ShortCut_D ----------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Anlage_Shortcut

Funktion lst ShortCut-Funktion aus. Damit werden Dimmer so eingestellt, wie
unter A programmiert.

bergabeparameter:      shortcut = [0..3] = Ziffern 0..3 fr A..D

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Anlage_Shortcut ( unsigned char shortcut )
{

  /* ----- lokale Variablen ----- */
  unsigned char result_value = false;
  unsigned char index;


   /* ----- Programmcode ----- */

   // Messagebox fr Demomodus anzeigen
   msgbox (30, BC_nobuttons, "\n   Sende Daten   \n    an Dimmer    \n");

   // BOOL StarLight_DimmerSchalten ( unsigned char dimmer, unsigned char status, unsigned char dimmwert )

   // Daten auslesen und Dimmer steuern
   for ( index = 0; index < 8; index++ )
   {
      result_value = StarLight_DimmerSchalten ( ( index + 1 ), starlightSetting.data[ shortcut ][ index ].dimmerstatus, starlightSetting.data[ shortcut ][ index ].dimmwert );

      // prfen ob Fehler zurckgemeldet wurde.
      if ( result_value != true )
      {
         // JA, dann Schleife abbrechen
         break;
      } // Fehlerbehandlung
   } // Schleife ber alle Dimmer

   // Auf jeden Fall Empfangsbuffer leeren
   RfStarLightPurgeBuffer( );

}
/* ----- Ende StarLight_Anlage_Shortcut ------------------------------------ */



/* ----------------------------------------------------------------------------
Funktion: StarLight_DrawDimmerGUI

Funktion zeichnet die grafische Darstellung der Beleuchtungsanlage

bergabeparameter:      dimmerData = aktiver Dimme

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DrawDimmerGUI( unsigned int dimmerData )
{
   /* ----- lokale Variablen ----- */
   char           result_string[ 255 ];

   /* ----- Programmcode ----- */

   // Display lschen
   lcd_fill ( 0 );

   // Dialog aufbauen
   set_font ( SMALLFONT );

   // Header ausgeben
   sprintf( result_string, "Dimmer %d", dimmerData );
   draw_string ( STARLIGHT_DIMMER_POS_X, STARLIGHT_DIMMER_POS_Y, result_string, LCD_COLOR_B, DRAW_PUT);

   switch ( starlightData.dimmer[ dimmerData - 1 ].status )
   {
      case STARLIGHT_FUNKTION_AUS:
         draw_string ( STARLIGHT_DIMMER_POS_X + 50, STARLIGHT_DIMMER_POS_Y, "AUS", LCD_COLOR_B, DRAW_PUT);
         break;
      case STARLIGHT_FUNKTION_EIN:
         draw_string ( STARLIGHT_DIMMER_POS_X + 50, STARLIGHT_DIMMER_POS_Y, "EIN", LCD_COLOR_B, DRAW_PUT);
         break;
   }

   FormatedDez( result_string, starlightData.dimmer[ dimmerData - 1  ].dimmwert, 2 );
   draw_string ( STARLIGHT_DIMMER_POS_X + 80, STARLIGHT_DIMMER_POS_Y, result_string, LCD_COLOR_B, DRAW_PUT);


   // Rahmen zeichnen
   draw_rect ( STARLIGHT_GUI_RAHMEN_X, STARLIGHT_GUI_RAHMEN_Y,
               STARLIGHT_GUI_RAHMEN_W, STARLIGHT_GUI_RAHMEN_H,
               STARLIGHT_GUI_RAHMEN_S,
               LCD_COLOR_B, DRAW_PUT );

   // Jeder Dimmerzweig kann selectiert bzw. nicht selektiert sein
   // Abhngig davon werden unterschiedliche Symbole ausgegeben

   // Dimmer 1 - Esstisch
   if ( dimmerData == STARLIGHT_DIMMER_1 )
   {
      // Lampen fr Dimmer 1 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER1_X - ( STARLIGHT_LAMPE_EIN_H_X / 2 ),
                 STARLIGHT_GUI_DIMMER1_Y - ( STARLIGHT_LAMPE_EIN_H_Y / 2 ),
                 &iconLampe_ein_h, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 1 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER1_X + 1 - ( STARLIGHT_LAMPE_AUS_H_X / 2 ),
                 STARLIGHT_GUI_DIMMER1_Y - ( STARLIGHT_LAMPE_AUS_H_Y / 2 ),
                 &iconLampe_aus_h, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 2 - Beleuchtungsreihe Fenster
   if ( dimmerData == STARLIGHT_DIMMER_2 )
   {
      // Lampen fr Dimmer 2 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y1 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y2 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y3 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y4 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 2 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y1 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y2 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y3 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER2_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER2_Y4 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 3 - Sternenhimmel
   if ( dimmerData == STARLIGHT_DIMMER_3 )
   {
      // Rahmen anstelle einzelner Lmpchen zeichnen
      draw_rect ( STARLIGHT_GUI_DIMMER3_X, STARLIGHT_GUI_DIMMER3_Y,
                  STARLIGHT_GUI_DIMMER3_W, STARLIGHT_GUI_DIMMER3_H,
                  STARLIGHT_GUI_DIMMER3_SON,
                  LCD_COLOR_B, DRAW_PUT );
   }
   else
   {
      // Rahmen anstelle einzelner Lmpchen zeichnen
      draw_rect ( STARLIGHT_GUI_DIMMER3_X, STARLIGHT_GUI_DIMMER3_Y,
                  STARLIGHT_GUI_DIMMER3_W, STARLIGHT_GUI_DIMMER3_H,
                  STARLIGHT_GUI_DIMMER3_SOFF,
                  LCD_COLOR_B, DRAW_PUT );
   }

   // Dimmer 4 - Durchgangsbeleuchtung
   if ( dimmerData == STARLIGHT_DIMMER_4 )
   {
      drawIcon ( STARLIGHT_GUI_DIMMER4_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER4_Y1 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER4_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER4_Y2 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 4 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER4_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER4_Y1 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER4_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER4_Y2 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 5 - Bilder
   if ( dimmerData == STARLIGHT_DIMMER_5 )
   {
      // Lampen fr Dimmer 5 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER5_X - ( STARLIGHT_BILDERLAMPE_X / 2 ),
                 STARLIGHT_GUI_DIMMER5_Y1 - ( STARLIGHT_BILDERLAMPE_Y / 2 ),
                 &iconBilderlampe, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER5_X - ( STARLIGHT_BILDERLAMPE_X / 2 ),
                 STARLIGHT_GUI_DIMMER5_Y2 - ( STARLIGHT_BILDERLAMPE_Y / 2 ),
                 &iconBilderlampe, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 5 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER5_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER5_Y1 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER5_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER5_Y2 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 6 - Wohnzimmertische
   if ( dimmerData == STARLIGHT_DIMMER_6 )
   {
      // Lampen fr Dimmer 6 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER6_X - ( STARLIGHT_LAMPE_EIN_V_X / 2 ),
                 STARLIGHT_GUI_DIMMER6_Y - ( STARLIGHT_LAMPE_EIN_V_Y / 2 ),
                 &iconLampe_ein_v, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 6 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER6_X - ( STARLIGHT_LAMPE_AUS_V_X / 2 ),
                 STARLIGHT_GUI_DIMMER6_Y - ( STARLIGHT_LAMPE_AUS_V_Y / 2 ),
                 &iconLampe_aus_v, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 7 - Regalbeleuchtung
   if ( dimmerData == STARLIGHT_DIMMER_7 )
   {
      // Lampen fr Dimmer 7 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER7_X1 - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER7_X2 - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER7_X3 - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 7 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER7_X1 - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER7_X2 - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER7_X3 - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER7_Y - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
   }

   // Dimmer 8 - Beleuchtungsreihe Wand
   if ( dimmerData == STARLIGHT_DIMMER_8 )
   {
      // Lampen fr Dimmer 8 in eingeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y1 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y2 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_EIN_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y3 - ( STARLIGHT_LICHT_EIN_Y / 2 ),
                 &iconLicht_an, LCD_COLOR_B, DRAW_PUT);
   }
   else
   {
      // Lampen fr Dimmer 8 in ausgeschaltetem Zustand zeichnen
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y1 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y2 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
      drawIcon ( STARLIGHT_GUI_DIMMER8_X - ( STARLIGHT_LICHT_AUS_X / 2 ),
                 STARLIGHT_GUI_DIMMER8_Y3 - ( STARLIGHT_LICHT_AUS_Y / 2 ),
                 &iconLicht_aus, LCD_COLOR_B, DRAW_PUT);
   }
}
/* ----- Ende StarLight_DrawDimmerGUI -------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_DimmerGUI

Funktion bedient ber grafisches Interface die Dimmer. Hierbei knnen die einzel
nen Dimmergruppen 1..8 ber Tasten angewhlt und bedient werden.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_DimmerGUI ( void )
{

   // Bemerkung: dimmer_index_GUI ist globale Variable fr Dimmer-Auswahl

   /* ----- lokale Variablen ----- */
   unsigned char  result_value;


   /* ----- Programmcode ----- */

   // Timer fr Men aufziehen
   addTimeout ( &menu_timer, MENU_SEQUENCE_TIMEOUT );

   // Grafik aufbauen
   StarLight_DrawDimmerGUI( dimmer_index_GUI );

   while (KEY_Exit) {};
   waitKeyUp ();
   while (1)
   {

      update_active_ind ();

      //waitKeyUpDown ();
      // Wait until key pressed instead.
      cpu_idle ();
      if (isKeyPressed ())
      {
         setBacklight (BL_AUTO);    // pwm value

         addTimeout ( &menu_timer, MENU_SEQUENCE_TIMEOUT );

         // Credits ausgeben
         if (KEY_Info)
         {
            // Grafik erneut aufbauen
            Starlight_Credits (  );
         }

         // Dialog verlassen
         else if ( KEY_Exit )
         {

            // Timeout aus Scheduler-Queue lschen
            removeTimeout( &menu_timer );
            menu_timer = 0;

            // Men verlassen
            return;
         }

         // Dimmerindex hochzhlen
         else if ( KEY_Up || KEY_Right )
         {
            dimmer_index_GUI++;

            // Wenn Max. Anzahl erreicht wieder von vorne anfangen
            if ( dimmer_index_GUI > STARLIGHT_DIMMER_ANZAHL )
            {
               dimmer_index_GUI = 1;
            }
         }

         // Dimmerindex runterzhlen
         else if ( KEY_Down || KEY_Left )
         {
            dimmer_index_GUI--;

            // Wenn bei erstem Dimmer angekommen auf letzten Dimmer schalten
            if ( dimmer_index_GUI < 1 )
            {
               dimmer_index_GUI = STARLIGHT_DIMMER_ANZAHL;
            }
         }

         // Dimmwert erhhen und Update gleich an Dimmer senden
         else if ( KEY_Vplus )
         {
            starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert++;
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert > STARLIGHT_DIMMER_DATA_MAX )
            {
               starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MAX;
            }
            result_value = StarLight_LampenDimmen ( dimmer_index_GUI, starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert );
         }

         // Dimmwert verringern und Update gleich an Dimmer senden
         else if ( KEY_Vminus )
         {
            starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert--;
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert < STARLIGHT_DIMMER_DATA_MIN )
            {
               starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert = STARLIGHT_DIMMER_DATA_MIN;
            }
            result_value = StarLight_LampenDimmen ( dimmer_index_GUI, starlightData.dimmer[ dimmer_index_GUI - 1 ].dimmwert );
         }

         // Dimmer ausschalten
         else if ( KEY_Pminus )
         {
            // Prfen ob Dimmer noch aus (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].status == STARLIGHT_FUNKTION_EIN )
            {
               // JA, dann auf jeden Fall Dimmer einschalten
               result_value = StarLight_LampenSchalten ( dimmer_index_GUI, STARLIGHT_FUNKTION_AUS );
            }
         }

         // Dimmer einschalten
         else if ( KEY_Pplus )
         {
            // Prfen ob Dimmer noch aus (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].status == STARLIGHT_FUNKTION_AUS )
            {
               // JA, dann auf jeden Fall Dimmer einschalten
               result_value = StarLight_LampenSchalten ( dimmer_index_GUI, STARLIGHT_FUNKTION_EIN );
            }
         }

         // Dimmer EIN und 33%
         else if (KEY_VTX1)
         {
            StarLight_DimmerSchalten( dimmer_index_GUI, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_33);
         }

         // Dimmer EIN und 66%
         else if (KEY_VTX2)
         {
            StarLight_DimmerSchalten( dimmer_index_GUI, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_66);
         }

         // Dimmer EIN und 100%
         else if (KEY_VTX3)
         {
            StarLight_DimmerSchalten( dimmer_index_GUI, STARLIGHT_FUNKTION_EIN, STARLIGHT_DIMMER_100);
         }

         // Programm A abrufen
         else if (KEY_A)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_A );
         }

         // Programm B abrufen
         else if (KEY_B)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_B );
         }

         // Programm C abrufen
         else if (KEY_C)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_C );
         }

         // Programm D abrufen
         else if (KEY_D)
         {
            StarLight_Anlage_Shortcut( STARLIGHT_SHORTCUT_KEY_D );
         }

         // Dimmer umschalten
         else if ( KEY_1 || KEY_2 || KEY_3 || KEY_4 || KEY_5 || KEY_6 || KEY_7 || KEY_8 )
         {
            // KEY's 1-8 auswerten
            if ( KEY_1 )      { dimmer_index_GUI = STARLIGHT_DIMMER_1; }
            else if ( KEY_2 ) { dimmer_index_GUI = STARLIGHT_DIMMER_2; }
            else if ( KEY_3 ) { dimmer_index_GUI = STARLIGHT_DIMMER_3; }
            else if ( KEY_4 ) { dimmer_index_GUI = STARLIGHT_DIMMER_4; }
            else if ( KEY_5 ) { dimmer_index_GUI = STARLIGHT_DIMMER_5; }
            else if ( KEY_6 ) { dimmer_index_GUI = STARLIGHT_DIMMER_6; }
            else if ( KEY_7 ) { dimmer_index_GUI = STARLIGHT_DIMMER_7; }
            else if ( KEY_8 ) { dimmer_index_GUI = STARLIGHT_DIMMER_8; }
         }

         // Lampe EIN des aktuellen Dimmers
         else if (KEY_Green )
         {
            // Prfen ob Dimmer noch aus (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].status == STARLIGHT_FUNKTION_AUS )
            {
               // JA, dann auf jeden Fall Dimmer einschalten
               result_value = StarLight_LampenSchalten ( dimmer_index_GUI, STARLIGHT_FUNKTION_EIN );
            }
         }

         // Lampe AUS des aktuellen Dimmers
         else if (KEY_Red)
         {
            // Prfen ob Dimmer noch ein (fr Optimierung damit nicht unntig Daten bertragen werden mssen)
            if ( starlightData.dimmer[ dimmer_index_GUI - 1 ].status == STARLIGHT_FUNKTION_EIN )
            {
               // JA, dann auf jeden Fall Dimmer ausschalten
               result_value = StarLight_LampenSchalten ( dimmer_index_GUI, STARLIGHT_FUNKTION_AUS );
            }
         }

         // Anzeige aller Dimmerdaten
         else if (KEY_TV)
         {
            StarLight_ShowDimmerData( );
         }

         // Anlage AUS/AN schalten
         else if (KEY_Power)
         {
            StarLight_AnlageSchalten(  );
         }

         // Tastenpaar fr ALLE Lampen EIN/AUS
         else if ( KEY_Yellow || KEY_Blue )
         {
            // Steuerung fr alle Lampen EIN/AUS
            if ( KEY_Yellow )
            {
               StarLight_AlleLampenSchalten ( FKT_STATUS_EIN );
            }
            else if ( KEY_Blue )
            {
               StarLight_AlleLampenSchalten ( FKT_STATUS_AUS );
            }
         }

         // Dimmer Einstellmen aufrufen
         else if (KEY_Menu)
         {
            menu_exec (&starlightSettingsMenu);
         }

         // Sound schalten
         else if (KEY_Mute)
         {
            StarLight_SoundSchalten (  );
         }

         // Betty Men anzeigen
         else if (KEY_Betty)
         {
            menu_exec (&mainMenu);
         }


         // Warten bis Taste losgelassen
         waitKeyUp ();

         // Display refresh
         StarLight_DrawDimmerGUI( dimmer_index_GUI );

      }   // if (isKeyPressed ())

      if ( !menu_timer )
      {
         // Timeout aus Scheduler-Queue lschen
         removeTimeout( &menu_timer );
         menu_timer = 0;

         // Men verlassen
         return;
      }
   }   // while (1)

}
/* ----- Ende StarLight_Dimmer_GUI ----------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLight_Check_Uebertemperatur

Funktion prft alle 8 Dimmer auf bertemperatur und gibt Statusbildschirm aus

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void StarLight_Check_Uebertemperatur ( void )
{

  /* ----- lokale Variablen ----- */
  char            temp_string[255];
  unsigned char   index;
  unsigned char   temp_flag = false;


   /* ----- Programmcode ----- */

   // Prfung ob Temperaturwert schon vorhanden und ob ausgewertet werden kann
   if ( starlightUebertemperatur_flag == false )
   {
      // NEIN, also Rcksprung
      return;
   }  // Rcksprung notwendig

   // Schleife ber alle 8 Dimmer zur Prfung
   for ( index = 0 ; index < STARLIGHT_DIMMER_ANZAHL ; index++ )
   {
      // Prfung
      if ( starlightData.dimmer[ index ].temperatur > starlightUebertemperatur )
      {
         temp_flag = true;
         break;
      }  // Prfung
   }  // Schleife ber alle 8 Dimmer

   // Prfung ob bertemperatur gemeldet
   if ( temp_flag == true )
   {
      // Messagebox anzeigen
      sprintf ( temp_string , "\n    Dimmer %d meldet    \n   Uebertemperatur   \n", (index + 1 ) );
      msgbox (30, BC_OKOnly, temp_string );

      // Dimmerdaten anzeigen
      StarLight_ShowDimmerData(  );

      // Temperaturwert zurcksetzen
      starlightUebertemperatur_flag = false;

   }  // bertemperatur

}
/* ----- Ende StarLight_Check_Uebertemperatur ------------------------------ */


/* ============================ ENDE ======================================= */
